@@ -4,7 +4,7 @@
 # also regenerates all aclocal.m4, config.h.in, Makefile.in, configure files
 # with new versions of autoconf or automake.
 #
-# This script requires autoconf-2.64..2.72 and automake-1.13..1.17 in the PATH.
+# This script requires autoconf-2.64..2.72 and automake-1.13..1.18 in the PATH.
 
 # Copyright (C) 2003-2025 Free Software Foundation, Inc.
 #
@@ -21,9 +21,11 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-# Prerequisite (if not used from a released tarball): either
-#   - the GNULIB_SRCDIR environment variable pointing to a gnulib checkout, or
-#   - a preceding invocation of './autopull.sh'.
+# Prerequisite (if not used from a released tarball):
+#   - A preceding invocation of './autopull.sh'.
+#   - If the GNULIB_SRCDIR environment variable was set during the
+#     './autopull.sh' invocation, pointing to a gnulib checkout, it should
+#     still be set now, during the './autogen.sh' invocation.
 #
 # Usage: ./autogen.sh [--skip-gnulib]
 #
@@ -99,25 +101,36 @@ if ! $skip_gnulib; then
     basename-lgpl
     binary-io
     bool
+    c-ctype
+    c-strtold
     closeout
     error
-    getopt-gnu
+    fzprintf-posix
     gettext-h
     havelib
+    mbrtoc32
+    mbszero
     memmove
     noreturn
+    options
     progname
     propername
+    quote
     relocatable-prog
     setlocale
     sigpipe
+    stdint-h
     stdio-h
     stdlib-h
+    strtoimax
+    strtold
     strtoul
+    strtoumax
     unistd-h
     unlocked-io
     xalloc
     xstring-buffer
+    xstrtold
   '
   GNULIB_MODULES_RUNTIME_OTHER='
     gettext-runtime-misc
@@ -205,7 +218,6 @@ if ! $skip_gnulib; then
     gcd
     getaddrinfo
     getline
-    getopt-gnu
     getrusage
     gettext-h
     gocomp-script
@@ -234,6 +246,7 @@ if ! $skip_gnulib; then
     open
     opendir
     openmp-init
+    options
     pipe-filter-ii
     progname
     propername
@@ -313,34 +326,118 @@ if ! $skip_gnulib; then
   '
   # Common dependencies of GNULIB_MODULES_TOOLS_FOR_SRC and GNULIB_MODULES_TOOLS_FOR_LIBGREP.
   GNULIB_MODULES_TOOLS_FOR_SRC_COMMON_DEPENDENCIES='
+    absolute-header
+    alignasof
     alloca-opt
+    assert-h
+    attribute
+    basename-lgpl
     bool
+    btowc
+    builtin-expect
+    c99
+    calloc-gnu
+    calloc-posix
+    cloexec
+    close
+    double-slash-root
+    dup2
+    errno-h
+    error
+    error-h
+    exitfail
     extensions
+    extensions-aix
+    extern-inline
+    fcntl
+    fcntl-h
+    fd-hook
+    filename
+    flexmember
+    fstat
+    gen-header
+    getdtablesize
+    getprogname
     gettext-h
+    gnulib-i18n
+    hard-locale
+    ialloc
+    idx
     include_next
-    locale-h
+    intprops
+    inttypes-h-incomplete
+    iswblank
+    iswctype
+    iswdigit
+    iswpunct
+    iswxdigit
+    largefile
+    libc-config
+    limits-h
     localcharset
+    localeconv
+    locale-h
+    lock
+    lstat
+    malloca
+    malloc-gnu
     malloc-posix
     mbrtowc
     mbsinit
+    mbszero
+    mbtowc
+    memchr
+    memcmp
+    memmove
+    minmax
+    msvc-inval
+    msvc-nothrow
     multiarch
+    obstack
+    once
+    open
+    pathmax
+    pthread-h
+    pthread-once
+    reallocarray
+    realloc-posix
+    sched-h
     setlocale-null
+    setlocale-null-unlocked
     snippet/arg-nonnull
     snippet/c++defs
+    snippet/_Noreturn
     snippet/warn-on-use
     ssize_t
+    stat
+    stat-time
+    stdckdint-h
     stddef-h
+    std-gnu11
     stdint-h
+    stdio-h
     stdlib-h
     streq
+    strerror
+    strerror-override
+    string-h
+    sys_stat-h
+    sys_types-h
+    threadlib
+    time-h
     unistd-h
+    vararrays
     verify
     wchar-h
+    wctype
     wctype-h
     windows-mutex
     windows-once
     windows-recmutex
     windows-rwlock
+    xalloc
+    xalloc-die
+    xalloc-oversized
   '
   GNULIB_MODULES_TOOLS_OTHER='
     gettext-tools-misc
@@ -378,6 +475,7 @@ if ! $skip_gnulib; then
   $GNULIB_TOOL --copy-file m4/libtextstyle.m4 gettext-tools/gnulib-m4/libtextstyle.m4 || exit $?
   # In gettext-tools/libgrep:
   GNULIB_MODULES_TOOLS_FOR_LIBGREP='
+    kwset
     mbrlen
     regex
   '
@@ -514,7 +612,7 @@ if ! $skip_gnulib; then
   done
 
   # Fetch INSTALL.generic.
-  $GNULIB_TOOL --copy-file doc/INSTALL.UTF-8 INSTALL.generic
+  $GNULIB_TOOL --copy-file doc/INSTALL.UTF-8 INSTALL.generic || exit $?
 fi
 
 # Make sure we get new versions of files brought in by automake.
@@ -548,6 +646,16 @@ aclocal -I ../../m4 -I ../m4 -I gnulib-m4 \
   || exit $?
 cd "$dir0"
 
+echo "$0: copying common files to gettext-runtime..."
+cp -p gettext-tools/wizard/po-templates/traditional/Makefile.in.in gettext-runtime/po/Makefile.in.in
+cp -p gettext-tools/wizard/po-templates/traditional/Rules-quot gettext-runtime/po/Rules-quot
+cp -p gettext-tools/wizard/po-templates/traditional/boldquot.sed gettext-runtime/po/boldquot.sed
+cp -p gettext-tools/wizard/po-templates/traditional/quot.sed gettext-runtime/po/quot.sed
+cp -p gettext-tools/wizard/po-templates/traditional/en@quot.header gettext-runtime/po/en@quot.header
+cp -p gettext-tools/wizard/po-templates/traditional/en@boldquot.header gettext-runtime/po/en@boldquot.header
+cp -p gettext-tools/wizard/po-templates/traditional/insert-header.sed gettext-runtime/po/insert-header.sed
+cp -p gettext-tools/wizard/po-templates/traditional/remove-potcdate.sed gettext-runtime/po/remove-potcdate.sed
+
 echo "$0: generating configure in gettext-runtime..."
 cd gettext-runtime
 aclocal -I m4 -I ../m4 -I gnulib-m4 \
@@ -576,16 +684,17 @@ aclocal -I ../../gettext-runtime/m4 -I ../../m4 \
   || exit $?
 cd "$dir0"
 
-echo "$0: copying common files from gettext-runtime to gettext-tools..."
+echo "$0: copying common files to gettext-tools..."
 cp -p gettext-runtime/ABOUT-NLS gettext-tools/ABOUT-NLS
-cp -p gettext-runtime/po/Makefile.in.in gettext-tools/po/Makefile.in.in
-cp -p gettext-runtime/po/Rules-quot gettext-tools/po/Rules-quot
-cp -p gettext-runtime/po/boldquot.sed gettext-tools/po/boldquot.sed
-cp -p gettext-runtime/po/quot.sed gettext-tools/po/quot.sed
-cp -p gettext-runtime/po/en@quot.header gettext-tools/po/en@quot.header
-cp -p gettext-runtime/po/en@boldquot.header gettext-tools/po/en@boldquot.header
-cp -p gettext-runtime/po/insert-header.sed gettext-tools/po/insert-header.sed
-cp -p gettext-runtime/po/remove-potcdate.sed gettext-tools/po/remove-potcdate.sed
+cp -p gettext-tools/wizard/po-templates/traditional/Makefile.in.in gettext-tools/po/Makefile.in.in
+cp -p gettext-tools/wizard/po-templates/traditional/Rules-quot gettext-tools/po/Rules-quot
+cp -p gettext-tools/wizard/po-templates/traditional/boldquot.sed gettext-tools/po/boldquot.sed
+cp -p gettext-tools/wizard/po-templates/traditional/quot.sed gettext-tools/po/quot.sed
+cp -p gettext-tools/wizard/po-templates/traditional/en@quot.header gettext-tools/po/en@quot.header
+cp -p gettext-tools/wizard/po-templates/traditional/en@boldquot.header gettext-tools/po/en@boldquot.header
+cp -p gettext-tools/wizard/po-templates/traditional/insert-header.sed gettext-tools/po/insert-header.sed
+cp -p gettext-tools/wizard/po-templates/traditional/remove-potcdate.sed gettext-tools/po/remove-potcdate.sed
+cp -p gettext-tools/wizard/po-templates/traditional/remove-potcdate.sed gettext-tools/examples/po/remove-potcdate.sed
 
 echo "$0: generating configure in gettext-tools..."
 cd gettext-tools
