@@ -1,6 +1,5 @@
 /* Handle list of needed message catalogs
-   Copyright (C) 1995-2023 Free Software Foundation, Inc.
-   Written by Ulrich Drepper <drepper@gnu.org>, 1995.
+   Copyright (C) 1995-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
@@ -15,6 +14,8 @@
    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
+/* Written by Ulrich Drepper and Bruno Haible.  */
+
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -37,7 +38,7 @@
 
 /* Handle multi-threaded applications.  */
 #ifdef _LIBC
-# include <bits/libc-lock.h>
+# include <libc-lock.h>
 # define gl_rwlock_define_initialized __libc_rwlock_define_initialized
 # define gl_rwlock_rdlock __libc_rwlock_rdlock
 # define gl_rwlock_wrlock __libc_rwlock_wrlock
@@ -62,7 +63,6 @@ gl_rwlock_define_initialized (static, lock);
    the DIRNAME or WDIRNAME, LOCALE, and DOMAINNAME parameters with respect
    to the currently established bindings.  */
 struct loaded_l10nfile *
-internal_function
 _nl_find_domain (const char *dirname,
 #if defined _WIN32 && !defined __CYGWIN__
 		 const wchar_t *wdirname,
@@ -140,64 +140,64 @@ _nl_find_domain (const char *dirname,
   alias_value = _nl_expand_alias (locale);
   if (alias_value != NULL)
     {
-      locale = strdup (alias_value);
+      size_t len = strlen (alias_value) + 1;
+      locale = (char *) malloc (len);
       if (locale == NULL)
-	return NULL;
+        return NULL;
+
+      memcpy (locale, alias_value, len);
     }
 
   /* Now we determine the single parts of the locale name.  First
      look for the language.  Termination symbols are `_', '.', and `@'.  */
   mask = _nl_explode_name (locale, &language, &modifier, &territory,
 			   &codeset, &normalized_codeset);
-  if (mask == -1)
-    /* This means we are out of core.  */
-    return NULL;
-
-  /* We need to protect modifying the _NL_LOADED_DOMAINS data.  */
-  gl_rwlock_wrlock (lock);
-
-  /* Create all possible locale entries which might be interested in
-     generalization.  */
-  retval = _nl_make_l10nflist (&_nl_loaded_domains,
-			       dirname,
-			       dirname != NULL ? strlen (dirname) + 1 : 0,
-#if defined _WIN32 && !defined __CYGWIN__
-			       wdirname,
-			       wdirname != NULL ? wcslen (wdirname) + 1 : 0,
-#endif
-			       mask, language, territory,
-			       codeset, normalized_codeset, modifier,
-			       domainname, 1);
-
-  gl_rwlock_unlock (lock);
-
-  if (retval == NULL)
-    /* This means we are out of core.  */
-    goto out;
-
-  if (retval->decided <= 0)
-    _nl_load_domain (retval, domainbinding);
-  if (retval->data == NULL)
+  if (mask != -1) /* not out-of-memory? */
     {
-      int cnt;
-      for (cnt = 0; retval->successor[cnt] != NULL; ++cnt)
+      /* We need to protect modifying the _NL_LOADED_DOMAINS data.  */
+      gl_rwlock_wrlock (lock);
+
+      /* Create all possible locale entries which might be interested in
+         generalization.  */
+      retval = _nl_make_l10nflist (&_nl_loaded_domains,
+				   dirname,
+				   dirname != NULL ? strlen (dirname) + 1 : 0,
+#if defined _WIN32 && !defined __CYGWIN__
+				   wdirname,
+				   wdirname != NULL ? wcslen (wdirname) + 1 : 0,
+#endif
+				   mask, language, territory,
+				   codeset, normalized_codeset, modifier,
+				   domainname, 1);
+
+      gl_rwlock_unlock (lock);
+
+      if (retval != NULL) /* not out-of-memory? */
 	{
-	  if (retval->successor[cnt]->decided <= 0)
-	    _nl_load_domain (retval->successor[cnt], domainbinding);
-	  if (retval->successor[cnt]->data != NULL)
-	    break;
+	  if (retval->decided <= 0)
+	    _nl_load_domain (retval, domainbinding);
+	  if (retval->data == NULL)
+	    {
+	      int cnt;
+	      for (cnt = 0; retval->successor[cnt] != NULL; ++cnt)
+		{
+		  if (retval->successor[cnt]->decided <= 0)
+		    _nl_load_domain (retval->successor[cnt], domainbinding);
+		  if (retval->successor[cnt]->data != NULL)
+		    break;
+		}
+	    }
 	}
+
+      /* The space for normalized_codeset is dynamically allocated.  Free it.  */
+      if (mask & XPG_NORM_CODESET)
+        free ((void *) normalized_codeset);
     }
 
   /* The room for an alias was dynamically allocated.  Free it now.  */
   if (alias_value != NULL)
     free (locale);
 
-out:
-  /* The space for normalized_codeset is dynamically allocated.  Free it.  */
-  if (mask & XPG_NORM_CODESET)
-    free ((void *) normalized_codeset);
-
   return retval;
 }
 
@@ -205,7 +205,7 @@ out:
 #ifdef _LIBC
 /* This is called from iconv/gconv_db.c's free_mem, as locales must
    be freed before freeing gconv steps arrays.  */
-void __libc_freeres_fn_section
+void
 _nl_finddomain_subfreeres (void)
 {
   struct loaded_l10nfile *runp = _nl_loaded_domains;
