@@ -1,6 +1,5 @@
 /* Formatted output to strings, using POSIX/XSI format strings with positions.
-   Copyright (C) 2003, 2006-2007, 2009-2011, 2018, 2020-2023 Free Software Foundation, Inc.
-   Written by Bruno Haible <bruno@clisp.org>, 2003.
+   Copyright (C) 2003-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
@@ -15,6 +14,8 @@
    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
+/* Written by Bruno Haible.  */
+
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -58,15 +59,28 @@ char *alloca ();
 # define EOVERFLOW E2BIG
 #endif
 
-/* When building a DLL, we must export some functions.  Note that because
-   the functions are only defined for binary backward compatibility, we
-   don't need to use __declspec(dllimport) in any case.  */
-#if HAVE_VISIBILITY && BUILDING_DLL
-# define DLL_EXPORTED __attribute__((__visibility__("default")))
-#elif defined _MSC_VER && BUILDING_DLL
-# define DLL_EXPORTED __declspec(dllexport)
+/* When building a shared library, we must export some functions.
+   Note that because this is a .c file, not a .h file, we don't need to use
+   __declspec(dllimport) in any case.  */
+#if HAVE_VISIBILITY && BUILDING_LIBRARY
+# define SHLIB_EXPORTED __attribute__((__visibility__("default")))
+#elif defined _MSC_VER && BUILDING_LIBRARY
+/* When building with MSVC, exporting a symbol means that the object file
+   contains a "linker directive" of the form /EXPORT:symbol.  This can be
+   inspected through the "objdump -s --section=.drectve FILE" or
+   "dumpbin /directives FILE" commands.
+   The symbols from this file should be exported if and only if the object
+   file gets included in a DLL.  Libtool, on Windows platforms, defines
+   the C macro DLL_EXPORT (together with PIC) when compiling for a shared
+   library (called DLL under Windows) and does not define it when compiling
+   an object file meant to be linked statically into some executable.  */
+# if defined DLL_EXPORT
+#  define SHLIB_EXPORTED __declspec(dllexport)
+# else
+#  define SHLIB_EXPORTED
+# endif
 #else
-# define DLL_EXPORTED
+# define SHLIB_EXPORTED
 #endif
 
 #define STATIC static
@@ -108,10 +122,12 @@ char *alloca ();
 /* Users don't expect libintl_fprintf to be less POSIX compliant
    than the fprintf implementation provided by gnulib or - on mingw -
    the one provided by mingw libs when __USE_MINGW_ANSI_STDIO is in
-   effect.  */
+   effect.
+   This definition is necessary, so that libgnuintl.h can declare
+   the *printf functions with _INTL_ATTRIBUTE_FORMAT_PRINTF_STANDARD.  */
 #define USE_REPLACEMENT_CODE_ALWAYS 1
 
-DLL_EXPORTED
+SHLIB_EXPORTED
 int
 libintl_vfprintf (FILE *stream, const char *format, va_list args)
 {
@@ -140,7 +156,7 @@ libintl_vfprintf (FILE *stream, const char *format, va_list args)
     }
 }
 
-DLL_EXPORTED
+SHLIB_EXPORTED
 int
 libintl_fprintf (FILE *stream, const char *format, ...)
 {
@@ -153,14 +169,14 @@ libintl_fprintf (FILE *stream, const char *format, ...)
   return retval;
 }
 
-DLL_EXPORTED
+SHLIB_EXPORTED
 int
 libintl_vprintf (const char *format, va_list args)
 {
   return libintl_vfprintf (stdout, format, args);
 }
 
-DLL_EXPORTED
+SHLIB_EXPORTED
 int
 libintl_printf (const char *format, ...)
 {
@@ -173,7 +189,7 @@ libintl_printf (const char *format, ...)
   return retval;
 }
 
-DLL_EXPORTED
+SHLIB_EXPORTED
 int
 libintl_vsprintf (char *resultbuf, const char *format, va_list args)
 {
@@ -200,7 +216,7 @@ libintl_vsprintf (char *resultbuf, const char *format, va_list args)
     }
 }
 
-DLL_EXPORTED
+SHLIB_EXPORTED
 int
 libintl_sprintf (char *resultbuf, const char *format, ...)
 {
@@ -228,7 +244,7 @@ libintl_sprintf (char *resultbuf, const char *format, ...)
 #  define system_vsnprintf vsnprintf
 # endif
 
-DLL_EXPORTED
+SHLIB_EXPORTED
 int
 libintl_vsnprintf (char *resultbuf, size_t length, const char *format, va_list args)
 {
@@ -263,7 +279,7 @@ libintl_vsnprintf (char *resultbuf, size_t length, const char *format, va_list a
     }
 }
 
-DLL_EXPORTED
+SHLIB_EXPORTED
 int
 libintl_snprintf (char *resultbuf, size_t length, const char *format, ...)
 {
@@ -280,7 +296,7 @@ libintl_snprintf (char *resultbuf, size_t length, const char *format, ...)
 
 #if HAVE_ASPRINTF
 
-DLL_EXPORTED
+SHLIB_EXPORTED
 int
 libintl_vasprintf (char **resultp, const char *format, va_list args)
 {
@@ -298,7 +314,7 @@ libintl_vasprintf (char **resultp, const char *format, va_list args)
   return length;
 }
 
-DLL_EXPORTED
+SHLIB_EXPORTED
 int
 libintl_asprintf (char **resultp, const char *format, ...)
 {
@@ -347,7 +363,7 @@ libintl_asprintf (char **resultp, const char *format, ...)
 #  define system_vswprintf vswprintf
 # endif
 
-DLL_EXPORTED
+SHLIB_EXPORTED
 int
 libintl_vfwprintf (FILE *stream, const wchar_t *format, va_list args)
 {
@@ -379,7 +395,7 @@ libintl_vfwprintf (FILE *stream, const wchar_t *format, va_list args)
     }
 }
 
-DLL_EXPORTED
+SHLIB_EXPORTED
 int
 libintl_fwprintf (FILE *stream, const wchar_t *format, ...)
 {
@@ -392,14 +408,14 @@ libintl_fwprintf (FILE *stream, const wchar_t *format, ...)
   return retval;
 }
 
-DLL_EXPORTED
+SHLIB_EXPORTED
 int
 libintl_vwprintf (const wchar_t *format, va_list args)
 {
   return libintl_vfwprintf (stdout, format, args);
 }
 
-DLL_EXPORTED
+SHLIB_EXPORTED
 int
 libintl_wprintf (const wchar_t *format, ...)
 {
@@ -412,7 +428,7 @@ libintl_wprintf (const wchar_t *format, ...)
   return retval;
 }
 
-DLL_EXPORTED
+SHLIB_EXPORTED
 int
 libintl_vswprintf (wchar_t *resultbuf, size_t length, const wchar_t *format, va_list args)
 {
@@ -453,7 +469,7 @@ libintl_vswprintf (wchar_t *resultbuf, size_t length, const wchar_t *format, va_
     }
 }
 
-DLL_EXPORTED
+SHLIB_EXPORTED
 int
 libintl_swprintf (wchar_t *resultbuf, size_t length, const wchar_t *format, ...)
 {
