@@ -1,6 +1,5 @@
 /* Substitution of environment variables in shell format strings.
-   Copyright (C) 2003-2024 Free Software Foundation, Inc.
-   Written by Bruno Haible <bruno@clisp.org>, 2003.
+   Copyright (C) 2003-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,12 +14,11 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+/* Written by Bruno Haible.  */
+
+#include <config.h>
 
 #include <errno.h>
-#include <getopt.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -29,6 +27,7 @@
 #include <locale.h>
 
 #include <error.h>
+#include "options.h"
 #include "attribute.h"
 #include "noreturn.h"
 #include "closeout.h"
@@ -46,15 +45,6 @@
 /* If true, substitution shall be performed on all variables.  */
 static bool all_variables;
 
-/* Long options.  */
-static const struct option long_options[] =
-{
-  { "help", no_argument, NULL, 'h' },
-  { "variables", no_argument, NULL, 'v' },
-  { "version", no_argument, NULL, 'V' },
-  { NULL, 0, NULL, 0 }
-};
-
 /* Forward declaration of local functions.  */
 _GL_NORETURN_FUNC static void usage (int status);
 static void print_variables (const char *string);
@@ -64,13 +54,6 @@ static void subst_from_stdin (void);
 int
 main (int argc, char *argv[])
 {
-  /* Default values for command line options.  */
-  bool show_variables = false;
-  bool do_help = false;
-  bool do_version = false;
-
-  int opt;
-
   /* Set program name for message texts.  */
   set_program_name (argv[0]);
 
@@ -79,29 +62,47 @@ main (int argc, char *argv[])
 
   /* Set the text message domain.  */
   bindtextdomain (PACKAGE, relocate (LOCALEDIR));
+  bindtextdomain ("gnulib", relocate (GNULIB_LOCALEDIR));
   textdomain (PACKAGE);
 
   /* Ensure that write errors on stdout are detected.  */
   atexit (close_stdout);
 
+  /* Default values for command line options.  */
+  bool show_variables = false;
+  bool do_help = false;
+  bool do_version = false;
+
   /* Parse command line options.  */
-  while ((opt = getopt_long (argc, argv, "hvV", long_options, NULL)) != EOF)
-    switch (opt)
-    {
-    case '\0':          /* Long option.  */
-      break;
-    case 'h':
-      do_help = true;
-      break;
-    case 'v':
-      show_variables = true;
-      break;
-    case 'V':
-      do_version = true;
-      break;
-    default:
-      usage (EXIT_FAILURE);
-    }
+  BEGIN_ALLOW_OMITTING_FIELD_INITIALIZERS
+  static const struct program_option options[] =
+  {
+    { "help",      'h', no_argument },
+    { "variables", 'v', no_argument },
+    { "version",   'V', no_argument },
+  };
+  END_ALLOW_OMITTING_FIELD_INITIALIZERS
+  start_options (argc, argv, options, MOVE_OPTIONS_FIRST, 0);
+  {
+    int opt;
+    while ((opt = get_next_option ()) != -1)
+      switch (opt)
+        {
+        case '\0':          /* Long option with key == 0.  */
+          break;
+        case 'h':
+          do_help = true;
+          break;
+        case 'v':
+          show_variables = true;
+          break;
+        case 'V':
+          do_version = true;
+          break;
+        default:
+          usage (EXIT_FAILURE);
+        }
+  }
 
   /* Version information is requested.  */
   if (do_version)
@@ -114,7 +115,7 @@ License GPLv3+: GNU GPL version 3 or later <%s>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n\
 "),
-              "2003-2024", "https://gnu.org/licenses/gpl.html");
+              "2003-2026", "https://gnu.org/licenses/gpl.html");
       printf (_("Written by %s.\n"), proper_name ("Bruno Haible"));
       exit (EXIT_SUCCESS);
     }
@@ -222,11 +223,11 @@ of the environment variables that are referenced in SHELL-FORMAT, one per line.\
          email address for this package.  Please add _another line_ saying
          "Report translation bugs to <...>\n" with the address for translation
          bugs (typically your translation team's web or email address).  */
-      printf(_("\
+      printf (_("\
 Report bugs in the bug tracker at <%s>\n\
 or by email to <%s>.\n"),
-             "https://savannah.gnu.org/projects/gettext",
-             "bug-gettext@gnu.org");
+              "https://savannah.gnu.org/projects/gettext",
+              "bug-gettext@gnu.org");
     }
 
   exit (status);
@@ -249,15 +250,12 @@ find_variables (const char *string,
   for (; *string != '\0';)
     if (*string++ == '$')
       {
-        const char *variable_start;
-        const char *variable_end;
-        bool valid;
-        char c;
-
         if (*string == '{')
           string++;
 
-        variable_start = string;
+        const char *variable_start = string;
+        char c;
+
         c = *string;
         if ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') || c == '_')
           {
@@ -265,8 +263,9 @@ find_variables (const char *string,
               c = *++string;
             while ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z')
                    || (c >= '0' && c <= '9') || c == '_');
-            variable_end = string;
+            const char *variable_end = string;
 
+            bool valid;
             if (variable_start[-1] == '{')
               {
                 if (*string == '}')
@@ -330,10 +329,8 @@ string_list_append (string_list_ty *slp, const char *s)
   /* Grow the list.  */
   if (slp->nitems >= slp->nitems_max)
     {
-      size_t nbytes;
-
       slp->nitems_max = slp->nitems_max * 2 + 4;
-      nbytes = slp->nitems_max * sizeof (slp->item[0]);
+      size_t nbytes = slp->nitems_max * sizeof (slp->item[0]);
       slp->item = (const char **) xrealloc (slp->item, nbytes);
     }
 
@@ -363,9 +360,7 @@ string_list_sort (string_list_ty *slp)
 MAYBE_UNUSED static inline int
 string_list_member (const string_list_ty *slp, const char *s)
 {
-  size_t j;
-
-  for (j = 0; j < slp->nitems; ++j)
+  for (size_t j = 0; j < slp->nitems; ++j)
     if (strcmp (slp->item[j], s) == 0)
       return 1;
   return 0;
@@ -375,10 +370,8 @@ string_list_member (const string_list_ty *slp, const char *s)
 static int
 sorted_string_list_member (const string_list_ty *slp, const char *s)
 {
-  size_t j1, j2;
-
-  j1 = 0;
-  j2 = slp->nitems;
+  size_t j1 = 0;
+  size_t j2 = slp->nitems;
   if (j2 > 0)
     {
       /* Binary search.  */
@@ -407,9 +400,7 @@ sorted_string_list_member (const string_list_ty *slp, const char *s)
 MAYBE_UNUSED static inline void
 string_list_destroy (string_list_ty *slp)
 {
-  size_t j;
-
-  for (j = 0; j < slp->nitems; ++j)
+  for (size_t j = 0; j < slp->nitems; ++j)
     free ((char *) slp->item[j]);
   if (slp->item != NULL)
     free (slp->item);
@@ -467,10 +458,10 @@ do_ungetc (int c)
 static void
 subst_from_stdin ()
 {
-  int c;
-
   for (;;)
     {
+      int c;
+
       c = do_getc ();
       if (c == EOF)
         break;
@@ -488,10 +479,8 @@ subst_from_stdin ()
             }
           if ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') || c == '_')
             {
-              struct string_buffer buffer;
-              bool valid;
-
               /* Accumulate the VARIABLE in buffer.  */
+              struct string_buffer buffer;
               sb_init (&buffer);
               do
                 {
@@ -501,6 +490,7 @@ subst_from_stdin ()
               while ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z')
                      || (c >= '0' && c <= '9') || c == '_');
 
+              bool valid;
               if (opening_brace)
                 {
                   if (c == '}')
@@ -548,7 +538,7 @@ subst_from_stdin ()
                   putchar ('$');
                   if (opening_brace)
                     putchar ('{');
-                  string_desc_fwrite (stdout, sb_contents (&buffer));
+                  sd_fwrite (stdout, sb_contents (&buffer));
                   if (closing_brace)
                     putchar ('}');
                 }
