@@ -1,5 +1,5 @@
 /* Formatted output with a POSIX compatible format string.
-   Copyright (C) 2025 Free Software Foundation, Inc.
+   Copyright (C) 2025-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
-/* Written by Bruno Haible <bruno@clisp.org>, 2025.  */
+/* Written by Bruno Haible.  */
 
 #include <config.h>
 
@@ -441,9 +441,7 @@ apply_format_string (const struct format_string *fmts,
                      size_t argc, char *argv[])
 {
   size_t npieces = fmts->npieces;
-  size_t i;
-
-  for (i = 0; i < npieces; i++)
+  for (size_t i = 0; i < npieces; i++)
     {
       struct format_piece *piece = &fmts->pieces[i];
 
@@ -457,10 +455,10 @@ apply_format_string (const struct format_string *fmts,
       else
         {
           /* Convert and print an argument.  */
-          char *arg;
           char zero[2] = { '0', '\0' };
           char *empty = zero + 1;
 
+          char *arg;
           if (piece->arg_number - 1 < argc)
             arg = argv[piece->arg_number - 1];
           else
@@ -485,9 +483,9 @@ apply_format_string (const struct format_string *fmts,
                  multibyte character.  */
               {
                 mbstate_t state;
-                char32_t wc;
-
                 mbszero (&state);
+
+                char32_t wc;
                 size_t ret = mbrtoc32 (&wc, arg, strnlen1 (arg, MB_CUR_MAX), &state);
                 arg[(int) ret >= 0 ? ret : 1] = '\0';
               }
@@ -514,9 +512,9 @@ apply_format_string (const struct format_string *fmts,
                        Use the first first multibyte character, if arg starts
                        with a valid multibyte character.  */
                     mbstate_t state;
-                    char32_t wc;
-
                     mbszero (&state);
+
+                    char32_t wc;
                     size_t ret = mbrtoc32 (&wc, arg + 1, strnlen1 (arg + 1, MB_CUR_MAX), &state);
                     if ((int) ret > 0)
                       arg_value = wc;
@@ -582,9 +580,9 @@ apply_format_string (const struct format_string *fmts,
                        Use the first first multibyte character, if arg starts
                        with a valid multibyte character.  */
                     mbstate_t state;
-                    char32_t wc;
-
                     mbszero (&state);
+
+                    char32_t wc;
                     size_t ret = mbrtoc32 (&wc, arg + 1, strnlen1 (arg + 1, MB_CUR_MAX), &state);
                     if ((int) ret > 0)
                       arg_value = wc;
