@@ -1,5 +1,5 @@
 /* Test of spin locks for communication between threads and signal handlers.
-   Copyright (C) 2005, 2008-2025 Free Software Foundation, Inc.
+   Copyright (C) 2005, 2008-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -43,11 +43,10 @@ main (void)
   /* Check a spin-lock initialized through asyncsafe_spin_init.  */
   {
     asyncsafe_spinlock_t local_spin_lock;
-    int i;
 
     asyncsafe_spin_init (&local_spin_lock);
 
-    for (i = 0; i < 10; i++)
+    for (int i = 0; i < 10; i++)
       {
         sigset_t saved_set;
         asyncsafe_spin_lock (&local_spin_lock, false, &set, &saved_set);
