@@ -1,9 +1,9 @@
 /* Test of <signal.h> substitute.
-   Copyright (C) 2009-2019 Free Software Foundation, Inc.
+   Copyright (C) 2009-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -20,6 +20,9 @@
 
 #include <signal.h>
 
+/* Check that SIG2STR_MAX is defined to a positive integer.  */
+static_assert (0 < SIG2STR_MAX);
+
 /* Check for required types.  */
 struct
 {
