@@ -1,7 +1,5 @@
 /* Recode Serbian text from Cyrillic to Latin script.
-   Copyright (C) 2006-2007, 2009 Free Software Foundation, Inc.
-   Written by Danilo Šegan <danilo@gnome.org>, 2006,
-   and Bruno Haible <bruno@clisp.org>, 2006.
+   Copyright (C) 2006-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,9 +14,9 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+/* Written by Danilo Šegan and Bruno Haible.  */
+
+#include <config.h>
 
 /* Specification.  */
 #include "filters.h"
