@@ -1,7 +1,5 @@
 /* GNU gettext - internationalization aids
-   Copyright (C) 1997-2024 Free Software Foundation, Inc.
-
-   This file was written by Peter Miller <millerp@canb.auug.org.au>
+   Copyright (C) 1997-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,11 +14,10 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+/* Written by Peter Miller, Ulrich Drepper, and Bruno Haible.  */
+
+#include <config.h>
 
-#include <getopt.h>
 #include <limits.h>
 #include <locale.h>
 #include <stdio.h>
@@ -29,6 +26,7 @@
 #include <textstyle.h>
 
 #include <error.h>
+#include "options.h"
 #include "noreturn.h"
 #include "closeout.h"
 #include "dir-list.h"
@@ -63,40 +61,6 @@ static int force_po;
 /* Target encoding.  */
 static const char *to_code;
 
-/* Long options.  */
-static const struct option long_options[] =
-{
-  { "add-location", optional_argument, NULL, 'n' },
-  { "color", optional_argument, NULL, CHAR_MAX + 5 },
-  { "directory", required_argument, NULL, 'D' },
-  { "escape", no_argument, NULL, 'E' },
-  { "files-from", required_argument, NULL, 'f' },
-  { "force-po", no_argument, &force_po, 1 },
-  { "help", no_argument, NULL, 'h' },
-  { "indent", no_argument, NULL, 'i' },
-  { "no-escape", no_argument, NULL, 'e' },
-  { "no-location", no_argument, NULL, CHAR_MAX + 7 },
-  { "no-wrap", no_argument, NULL, CHAR_MAX + 2 },
-  { "omit-header", no_argument, NULL, CHAR_MAX + 1 },
-  { "output", required_argument, NULL, 'o' }, /* for backward compatibility */
-  { "output-file", required_argument, NULL, 'o' },
-  { "properties-input", no_argument, NULL, 'P' },
-  { "properties-output", no_argument, NULL, 'p' },
-  { "sort-by-file", no_argument, NULL, 'F' },
-  { "sort-output", no_argument, NULL, 's' },
-  { "strict", no_argument, NULL, 'S' },
-  { "stringtable-input", no_argument, NULL, CHAR_MAX + 3 },
-  { "stringtable-output", no_argument, NULL, CHAR_MAX + 4 },
-  { "style", required_argument, NULL, CHAR_MAX + 6 },
-  { "to-code", required_argument, NULL, 't' },
-  { "unique", no_argument, NULL, 'u' },
-  { "version", no_argument, NULL, 'V' },
-  { "width", required_argument, NULL, 'w' },
-  { "more-than", required_argument, NULL, '>' },
-  { "less-than", required_argument, NULL, '<' },
-  { NULL, 0, NULL, 0 }
-};
-
 
 /* Forward declaration of local functions.  */
 _GL_NORETURN_FUNC static void usage (int status);
@@ -105,19 +69,6 @@ _GL_NORETURN_FUNC static void usage (int status);
 int
 main (int argc, char *argv[])
 {
-  int cnt;
-  int optchar;
-  bool do_help = false;
-  bool do_version = false;
-  msgdomain_list_ty *result;
-  catalog_input_format_ty input_syntax = &input_format_po;
-  catalog_output_format_ty output_syntax = &output_format_po;
-  bool sort_by_msgid = false;
-  bool sort_by_filepos = false;
-  const char *files_from = NULL;
-  string_list_ty *file_list;
-  char *output_file = NULL;
-
   /* Set program name for messages.  */
   set_program_name (argv[0]);
   error_print_progname = maybe_print_progname;
@@ -135,146 +86,190 @@ main (int argc, char *argv[])
   /* Ensure that write errors on stdout are detected.  */
   atexit (close_stdout);
 
-  /* Set default values for variables.  */
+  /* Default values for command line options.  */
+  bool do_help = false;
+  bool do_version = false;
+  char *output_file = NULL;
+  const char *files_from = NULL;
   more_than = -1;
   less_than = -1;
   use_first = false;
+  catalog_input_format_ty input_syntax = &input_format_po;
+  catalog_output_format_ty output_syntax = &output_format_po;
+  bool sort_by_msgid = false;
+  bool sort_by_filepos = false;
 
-  while ((optchar = getopt_long (argc, argv, "<:>:D:eEf:Fhino:pPst:uVw:",
-                                 long_options, NULL)) != EOF)
-    switch (optchar)
-      {
-      case '\0':                /* Long option.  */
-        break;
-
-      case '>':
+  /* Parse command line options.  */
+  BEGIN_ALLOW_OMITTING_FIELD_INITIALIZERS
+  static const struct program_option options[] =
+  {
+    { "add-location",       CHAR_MAX + 'n', optional_argument },
+    { NULL,                 'n',            no_argument       },
+    { "color",              CHAR_MAX + 5,   optional_argument },
+    { "directory",          'D',            required_argument },
+    { "escape",             'E',            no_argument       },
+    { "files-from",         'f',            required_argument },
+    { "force-po",           0,              no_argument,      &force_po, 1 },
+    { "help",               'h',            no_argument       },
+    { "indent",             'i',            no_argument       },
+    { "no-escape",          'e',            no_argument       },
+    { "no-location",        CHAR_MAX + 7,   no_argument       },
+    { "no-wrap",            CHAR_MAX + 2,   no_argument       },
+    { "omit-header",        CHAR_MAX + 1,   no_argument       },
+    { "output",             'o',            required_argument }, /* for backward compatibility */
+    { "output-file",        'o',            required_argument },
+    { "properties-input",   'P',            no_argument       },
+    { "properties-output",  'p',            no_argument       },
+    { "sort-by-file",       'F',            no_argument       },
+    { "sort-output",        's',            no_argument       },
+    { "strict",             CHAR_MAX + 8,   no_argument       },
+    { "stringtable-input",  CHAR_MAX + 3,   no_argument       },
+    { "stringtable-output", CHAR_MAX + 4,   no_argument       },
+    { "style",              CHAR_MAX + 6,   required_argument },
+    { "to-code",            't',            required_argument },
+    { "unique",             'u',            no_argument       },
+    { "version",            'V',            no_argument       },
+    { "width",              'w',            required_argument },
+    { "more-than",          '>',            required_argument },
+    { "less-than",          '<',            required_argument },
+  };
+  END_ALLOW_OMITTING_FIELD_INITIALIZERS
+  start_options (argc, argv, options, MOVE_OPTIONS_FIRST, 0);
+  {
+    int optchar;
+    while ((optchar = get_next_option ()) != -1)
+      switch (optchar)
         {
-          int value;
-          char *endp;
-          value = strtol (optarg, &endp, 10);
-          if (endp != optarg)
-            more_than = value;
-        }
-        break;
+        case '\0':                /* Long option with key == 0.  */
+          break;
 
-      case '<':
-        {
-          int value;
-          char *endp;
-          value = strtol (optarg, &endp, 10);
-          if (endp != optarg)
-            less_than = value;
-        }
-        break;
+        case '>':
+          {
+            char *endp;
+            int value = strtol (optarg, &endp, 10);
+            if (endp != optarg)
+              more_than = value;
+          }
+          break;
 
-      case 'D':
-        dir_list_append (optarg);
-        break;
+        case '<':
+          {
+            char *endp;
+            int value = strtol (optarg, &endp, 10);
+            if (endp != optarg)
+              less_than = value;
+          }
+          break;
 
-      case 'e':
-        message_print_style_escape (false);
-        break;
+        case 'D':
+          dir_list_append (optarg);
+          break;
 
-      case 'E':
-        message_print_style_escape (true);
-        break;
+        case 'e':
+          message_print_style_escape (false);
+          break;
 
-      case 'f':
-        files_from = optarg;
-        break;
+        case 'E':
+          message_print_style_escape (true);
+          break;
 
-      case 'F':
-        sort_by_filepos = true;
-        break;
+        case 'f':
+          files_from = optarg;
+          break;
 
-      case 'h':
-        do_help = true;
-        break;
+        case 'F':
+          sort_by_filepos = true;
+          break;
 
-      case 'i':
-        message_print_style_indent ();
-        break;
+        case 'h':
+          do_help = true;
+          break;
 
-      case 'n':
-        if (handle_filepos_comment_option (optarg))
+        case 'i':
+          message_print_style_indent ();
+          break;
+
+        case 'n':            /* -n */
+        case CHAR_MAX + 'n': /* --add-location[={full|yes|file|never|no}] */
+          if (handle_filepos_comment_option (optarg))
+            usage (EXIT_FAILURE);
+          break;
+
+        case 'o':
+          output_file = optarg;
+          break;
+
+        case 'p':
+          output_syntax = &output_format_properties;
+          break;
+
+        case 'P':
+          input_syntax = &input_format_properties;
+          break;
+
+        case 's':
+          sort_by_msgid = true;
+          break;
+
+        case CHAR_MAX + 8: /* --strict */
+          message_print_style_uniforum ();
+          break;
+
+        case 't':
+          to_code = optarg;
+          break;
+
+        case 'u':
+          less_than = 2;
+          break;
+
+        case 'V':
+          do_version = true;
+          break;
+
+        case 'w':
+          {
+            char *endp;
+            int value = strtol (optarg, &endp, 10);
+            if (endp != optarg)
+              message_page_width_set (value);
+          }
+          break;
+
+        case CHAR_MAX + 1:
+          omit_header = true;
+          break;
+
+        case CHAR_MAX + 2: /* --no-wrap */
+          message_page_width_ignore ();
+          break;
+
+        case CHAR_MAX + 3: /* --stringtable-input */
+          input_syntax = &input_format_stringtable;
+          break;
+
+        case CHAR_MAX + 4: /* --stringtable-output */
+          output_syntax = &output_format_stringtable;
+          break;
+
+        case CHAR_MAX + 5: /* --color */
+          if (handle_color_option (optarg) || color_test_mode)
+            usage (EXIT_FAILURE);
+          break;
+
+        case CHAR_MAX + 6: /* --style */
+          handle_style_option (optarg);
+          break;
+
+        case CHAR_MAX + 7: /* --no-location */
+          message_print_style_filepos (filepos_comment_none);
+          break;
+
+        default:
           usage (EXIT_FAILURE);
-        break;
-
-      case 'o':
-        output_file = optarg;
-        break;
-
-      case 'p':
-        output_syntax = &output_format_properties;
-        break;
-
-      case 'P':
-        input_syntax = &input_format_properties;
-        break;
-
-      case 's':
-        sort_by_msgid = true;
-        break;
-
-      case 'S':
-        message_print_style_uniforum ();
-        break;
-
-      case 't':
-        to_code = optarg;
-        break;
-
-      case 'u':
-        less_than = 2;
-        break;
-
-      case 'V':
-        do_version = true;
-        break;
-
-      case 'w':
-        {
-          int value;
-          char *endp;
-          value = strtol (optarg, &endp, 10);
-          if (endp != optarg)
-            message_page_width_set (value);
+          /* NOTREACHED */
         }
-        break;
-
-      case CHAR_MAX + 1:
-        omit_header = true;
-        break;
-
-      case CHAR_MAX + 2: /* --no-wrap */
-        message_page_width_ignore ();
-        break;
-
-      case CHAR_MAX + 3: /* --stringtable-input */
-        input_syntax = &input_format_stringtable;
-        break;
-
-      case CHAR_MAX + 4: /* --stringtable-output */
-        output_syntax = &output_format_stringtable;
-        break;
-
-      case CHAR_MAX + 5: /* --color */
-        if (handle_color_option (optarg) || color_test_mode)
-          usage (EXIT_FAILURE);
-        break;
-
-      case CHAR_MAX + 6: /* --style */
-        handle_style_option (optarg);
-        break;
-
-      case CHAR_MAX + 7: /* --no-location */
-        message_print_style_filepos (filepos_comment_none);
-        break;
-
-      default:
-        usage (EXIT_FAILURE);
-        /* NOTREACHED */
-      }
+  }
 
   /* Version information requested.  */
   if (do_version)
@@ -287,7 +282,7 @@ License GPLv3+: GNU GPL version 3 or later <%s>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n\
 "),
-              "1995-2024", "https://gnu.org/licenses/gpl.html");
+              "1995-2026", "https://gnu.org/licenses/gpl.html");
       printf (_("Written by %s.\n"), proper_name ("Peter Miller"));
       exit (EXIT_SUCCESS);
     }
@@ -302,12 +297,13 @@ There is NO WARRANTY, to the extent permitted by law.\n\
            "--sort-output", "--sort-by-file");
 
   /* Determine list of files we have to process.  */
+  string_list_ty *file_list;
   if (files_from != NULL)
     file_list = read_names_from_file (files_from);
   else
     file_list = string_list_alloc ();
   /* Append names from command line.  */
-  for (cnt = optind; cnt < argc; ++cnt)
+  for (int cnt = optind; cnt < argc; ++cnt)
     string_list_append_unique (file_list, argv[cnt]);
 
   /* Test whether sufficient input files were given.  */
@@ -330,7 +326,8 @@ There is NO WARRANTY, to the extent permitted by law.\n\
   /* Read input files, then filter, convert and merge messages.  */
   allow_duplicates = true;
   msgcomm_mode = true;
-  result = catenate_msgdomain_list (file_list, input_syntax, to_code);
+  msgdomain_list_ty *result =
+    catenate_msgdomain_list (file_list, input_syntax, to_code);
 
   string_list_free (file_list);
 
@@ -465,7 +462,7 @@ Informative output:\n"));
          email address for this package.  Please add _another line_ saying
          "Report translation bugs to <...>\n" with the address for translation
          bugs (typically your translation team's web or email address).  */
-      printf(_("\
+      printf (_("\
 Report bugs in the bug tracker at <%s>\n\
 or by email to <%s>.\n"),
              "https://savannah.gnu.org/projects/gettext",
