@@ -1,6 +1,5 @@
 /* Edit translations using a subprocess.
-   Copyright (C) 2001-2010, 2012 Free Software Foundation, Inc.
-   Written by Bruno Haible <haible@clisp.cons.org>, 2001.
+   Copyright (C) 2001-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,14 +12,13 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
+
+/* Written by Bruno Haible.  */
 
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
+#include <config.h>
 
-#include <getopt.h>
 #include <limits.h>
 #include <locale.h>
 #include <stdio.h>
@@ -30,16 +28,20 @@
 #include <sys/time.h>
 #include <unistd.h>
 
+#include <textstyle.h>
+
+#include <error.h>
+#include "options.h"
+#include "noreturn.h"
 #include "closeout.h"
 #include "dir-list.h"
-#include "error.h"
 #include "xvasprintf.h"
 #include "error-progname.h"
 #include "progname.h"
 #include "relocatable.h"
-#include "basename.h"
+#include "basename-lgpl.h"
 #include "message.h"
-#include "read-catalog.h"
+#include "read-catalog-file.h"
 #include "read-po.h"
 #include "read-properties.h"
 #include "read-stringtable.h"
@@ -47,7 +49,6 @@
 #include "write-po.h"
 #include "write-properties.h"
 #include "write-stringtable.h"
-#include "color.h"
 #include "msgl-charset.h"
 #include "xalloc.h"
 #include "findprog.h"
@@ -55,6 +56,7 @@
 #include "xsetenv.h"
 #include "filters.h"
 #include "msgl-iconv.h"
+#include "xerror-handler.h"
 #include "po-charset.h"
 #include "propername.h"
 #include "gettext.h"
@@ -81,45 +83,14 @@ static const char *sub_path;
 static const char **sub_argv;
 static int sub_argc;
 
+static bool newline;
+
 /* Filter function.  */
 static void (*filter) (const char *str, size_t len, char **resultp, size_t *lengthp);
 
-/* Long options.  */
-static const struct option long_options[] =
-{
-  { "add-location", optional_argument, NULL, 'n' },
-  { "color", optional_argument, NULL, CHAR_MAX + 6 },
-  { "directory", required_argument, NULL, 'D' },
-  { "escape", no_argument, NULL, 'E' },
-  { "force-po", no_argument, &force_po, 1 },
-  { "help", no_argument, NULL, 'h' },
-  { "indent", no_argument, NULL, CHAR_MAX + 1 },
-  { "input", required_argument, NULL, 'i' },
-  { "keep-header", no_argument, &keep_header, 1 },
-  { "no-escape", no_argument, NULL, CHAR_MAX + 2 },
-  { "no-location", no_argument, NULL, CHAR_MAX + 8 },
-  { "no-wrap", no_argument, NULL, CHAR_MAX + 3 },
-  { "output-file", required_argument, NULL, 'o' },
-  { "properties-input", no_argument, NULL, 'P' },
-  { "properties-output", no_argument, NULL, 'p' },
-  { "sort-by-file", no_argument, NULL, 'F' },
-  { "sort-output", no_argument, NULL, 's' },
-  { "strict", no_argument, NULL, 'S' },
-  { "stringtable-input", no_argument, NULL, CHAR_MAX + 4 },
-  { "stringtable-output", no_argument, NULL, CHAR_MAX + 5 },
-  { "style", required_argument, NULL, CHAR_MAX + 7 },
-  { "version", no_argument, NULL, 'V' },
-  { "width", required_argument, NULL, 'w', },
-  { NULL, 0, NULL, 0 }
-};
-
 
 /* Forward declaration of local functions.  */
-static void usage (int status)
-#if defined __GNUC__ && ((__GNUC__ == 2 && __GNUC_MINOR__ >= 5) || __GNUC__ > 2)
-        __attribute__ ((noreturn))
-#endif
-;
+_GL_NORETURN_FUNC static void usage (int status);
 static void generic_filter (const char *str, size_t len, char **resultp, size_t *lengthp);
 static msgdomain_list_ty *process_msgdomain_list (msgdomain_list_ty *mdlp);
 
@@ -127,164 +98,194 @@ static msgdomain_list_ty *process_msgdomain_list (msgdomain_list_ty *mdlp);
 int
 main (int argc, char **argv)
 {
-  int opt;
-  bool do_help;
-  bool do_version;
-  char *output_file;
-  const char *input_file;
-  msgdomain_list_ty *result;
-  catalog_input_format_ty input_syntax = &input_format_po;
-  catalog_output_format_ty output_syntax = &output_format_po;
-  bool sort_by_filepos = false;
-  bool sort_by_msgid = false;
-  int i;
-
   /* Set program name for messages.  */
   set_program_name (argv[0]);
   error_print_progname = maybe_print_progname;
+  gram_max_allowed_errors = 20;
 
-#ifdef HAVE_SETLOCALE
   /* Set locale via LC_ALL.  */
   setlocale (LC_ALL, "");
-#endif
 
   /* Set the text message domain.  */
   bindtextdomain (PACKAGE, relocate (LOCALEDIR));
+  bindtextdomain ("gnulib", relocate (GNULIB_LOCALEDIR));
   bindtextdomain ("bison-runtime", relocate (BISON_LOCALEDIR));
   textdomain (PACKAGE);
 
   /* Ensure that write errors on stdout are detected.  */
   atexit (close_stdout);
 
-  /* Set default values for variables.  */
-  do_help = false;
-  do_version = false;
-  output_file = NULL;
-  input_file = NULL;
+  /* Default values for command line options.  */
+  bool do_help = false;
+  bool do_version = false;
+  char *output_file = NULL;
+  const char *input_file = NULL;
+  catalog_input_format_ty input_syntax = &input_format_po;
+  catalog_output_format_ty output_syntax = &output_format_po;
+  bool sort_by_filepos = false;
+  bool sort_by_msgid = false;
 
-  /* The '+' in the options string causes option parsing to terminate when
-     the first non-option, i.e. the subprogram name, is encountered.  */
-  while ((opt = getopt_long (argc, argv, "+D:EFhi:n:o:pPsVw:", long_options,
-                             NULL))
-         != EOF)
-    switch (opt)
-      {
-      case '\0':                /* Long option.  */
-        break;
-
-      case 'D':
-        dir_list_append (optarg);
-        break;
-
-      case 'E':
-        message_print_style_escape (true);
-        break;
-
-      case 'F':
-        sort_by_filepos = true;
-        break;
-
-      case 'h':
-        do_help = true;
-        break;
-
-      case 'i':
-        if (input_file != NULL)
-          {
-            error (EXIT_SUCCESS, 0, _("at most one input file allowed"));
-            usage (EXIT_FAILURE);
-          }
-        input_file = optarg;
-        break;
-
-      case 'n':
-        if (handle_filepos_comment_option (optarg))
-          usage (EXIT_FAILURE);
-        break;
-
-      case 'o':
-        output_file = optarg;
-        break;
-
-      case 'p':
-        output_syntax = &output_format_properties;
-        break;
-
-      case 'P':
-        input_syntax = &input_format_properties;
-        break;
-
-      case 's':
-        sort_by_msgid = true;
-        break;
-
-      case 'S':
-        message_print_style_uniforum ();
-        break;
-
-      case 'V':
-        do_version = true;
-        break;
-
-      case 'w':
+  /* Parse command line options.  */
+  BEGIN_ALLOW_OMITTING_FIELD_INITIALIZERS
+  static const struct program_option options[] =
+  {
+    { "add-location",       CHAR_MAX + 'n', optional_argument },
+    { NULL,                 'n',            no_argument       },
+    { "color",              CHAR_MAX + 6,   optional_argument },
+    { "directory",          'D',            required_argument },
+    { "escape",             'E',            no_argument       },
+    { "force-po",           0,              no_argument,      &force_po, 1 },
+    { "help",               'h',            no_argument       },
+    { "indent",             CHAR_MAX + 1,   no_argument       },
+    { "input",              'i',            required_argument },
+    { "keep-header",        0,              no_argument,      &keep_header, 1 },
+    { "newline",            CHAR_MAX + 9,   no_argument       },
+    { "no-escape",          CHAR_MAX + 2,   no_argument       },
+    { "no-location",        CHAR_MAX + 8,   no_argument       },
+    { "no-wrap",            CHAR_MAX + 3,   no_argument       },
+    { "output-file",        'o',            required_argument },
+    { "properties-input",   'P',            no_argument       },
+    { "properties-output",  'p',            no_argument       },
+    { "sort-by-file",       'F',            no_argument       },
+    { "sort-output",        's',            no_argument       },
+    { "strict",             CHAR_MAX + 10,  no_argument       },
+    { "stringtable-input",  CHAR_MAX + 4,   no_argument       },
+    { "stringtable-output", CHAR_MAX + 5,   no_argument       },
+    { "style",              CHAR_MAX + 7,   required_argument },
+    { "version",            'V',            no_argument       },
+    { "width",              'w',            required_argument },
+  };
+  END_ALLOW_OMITTING_FIELD_INITIALIZERS
+  /* The flag NON_OPTION_TERMINATES_OPTIONS causes option parsing to terminate
+     when the first non-option, i.e. the subprogram name, is encountered.  */
+  start_options (argc, argv, options, NON_OPTION_TERMINATES_OPTIONS, 0);
+  {
+    int opt;
+    while ((opt = get_next_option ()) != -1)
+      switch (opt)
         {
-          int value;
-          char *endp;
-          value = strtol (optarg, &endp, 10);
-          if (endp != optarg)
-            message_page_width_set (value);
-        }
-        break;
+        case '\0':                /* Long option with key == 0.  */
+          break;
 
-      case CHAR_MAX + 1:
-        message_print_style_indent ();
-        break;
+        case 'D':
+          dir_list_append (optarg);
+          break;
 
-      case CHAR_MAX + 2:
-        message_print_style_escape (false);
-        break;
+        case 'E':
+          message_print_style_escape (true);
+          break;
 
-      case CHAR_MAX + 3: /* --no-wrap */
-        message_page_width_ignore ();
-        break;
+        case 'F':
+          sort_by_filepos = true;
+          break;
 
-      case CHAR_MAX + 4: /* --stringtable-input */
-        input_syntax = &input_format_stringtable;
-        break;
+        case 'h':
+          do_help = true;
+          break;
 
-      case CHAR_MAX + 5: /* --stringtable-output */
-        output_syntax = &output_format_stringtable;
-        break;
+        case 'i':
+          if (input_file != NULL)
+            {
+              error (EXIT_SUCCESS, 0, _("at most one input file allowed"));
+              usage (EXIT_FAILURE);
+            }
+          input_file = optarg;
+          break;
 
-      case CHAR_MAX + 6: /* --color */
-        if (handle_color_option (optarg) || color_test_mode)
+        case 'n':            /* -n */
+        case CHAR_MAX + 'n': /* --add-location[={full|yes|file|never|no}] */
+          if (handle_filepos_comment_option (optarg))
+            usage (EXIT_FAILURE);
+          break;
+
+        case 'o':
+          output_file = optarg;
+          break;
+
+        case 'p':
+          output_syntax = &output_format_properties;
+          break;
+
+        case 'P':
+          input_syntax = &input_format_properties;
+          break;
+
+        case 's':
+          sort_by_msgid = true;
+          break;
+
+        case CHAR_MAX + 10: /* --strict */
+          message_print_style_uniforum ();
+          break;
+
+        case 'V':
+          do_version = true;
+          break;
+
+        case 'w':
+          {
+            char *endp;
+            int value = strtol (optarg, &endp, 10);
+            if (endp != optarg)
+              message_page_width_set (value);
+          }
+          break;
+
+        case CHAR_MAX + 1:
+          message_print_style_indent ();
+          break;
+
+        case CHAR_MAX + 2:
+          message_print_style_escape (false);
+          break;
+
+        case CHAR_MAX + 3: /* --no-wrap */
+          message_page_width_ignore ();
+          break;
+
+        case CHAR_MAX + 4: /* --stringtable-input */
+          input_syntax = &input_format_stringtable;
+          break;
+
+        case CHAR_MAX + 5: /* --stringtable-output */
+          output_syntax = &output_format_stringtable;
+          break;
+
+        case CHAR_MAX + 6: /* --color */
+          if (handle_color_option (optarg) || color_test_mode)
+            usage (EXIT_FAILURE);
+          break;
+
+        case CHAR_MAX + 7: /* --style */
+          handle_style_option (optarg);
+          break;
+
+        case CHAR_MAX + 8: /* --no-location */
+          message_print_style_filepos (filepos_comment_none);
+          break;
+
+        case CHAR_MAX + 9: /* --newline */
+          newline = true;
+          break;
+
+        default:
           usage (EXIT_FAILURE);
-        break;
-
-      case CHAR_MAX + 7: /* --style */
-        handle_style_option (optarg);
-        break;
-
-      case CHAR_MAX + 8: /* --no-location */
-        message_print_style_filepos (filepos_comment_none);
-        break;
-
-      default:
-        usage (EXIT_FAILURE);
-        break;
-      }
+          break;
+        }
+  }
 
   /* Version information is requested.  */
   if (do_version)
     {
-      printf ("%s (GNU %s) %s\n", basename (program_name), PACKAGE, VERSION);
+      printf ("%s (GNU %s) %s\n", last_component (program_name),
+              PACKAGE, VERSION);
       /* xgettext: no-wrap */
       printf (_("Copyright (C) %s Free Software Foundation, Inc.\n\
-License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n\
+License GPLv3+: GNU GPL version 3 or later <%s>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n\
 "),
-              "2001-2010");
+              "2001-2026", "https://gnu.org/licenses/gpl.html");
       printf (_("Written by %s.\n"), proper_name ("Bruno Haible"));
       exit (EXIT_SUCCESS);
     }
@@ -306,9 +307,12 @@ There is NO WARRANTY, to the extent permitted by law.\n\
   /* Build argument list for the program.  */
   sub_argc = argc - optind;
   sub_argv = XNMALLOC (sub_argc + 1, const char *);
-  for (i = 0; i < sub_argc; i++)
-    sub_argv[i] = argv[optind + i];
-  sub_argv[i] = NULL;
+  {
+    size_t i;
+    for (i = 0; i < sub_argc; i++)
+      sub_argv[i] = argv[optind + i];
+    sub_argv[i] = NULL;
+  }
 
   /* Extra checks for sed scripts.  */
   if (strcmp (sub_name, "sed") == 0)
@@ -318,7 +322,7 @@ There is NO WARRANTY, to the extent permitted by law.\n\
                _("at least one sed script must be specified"));
 
       /* Replace GNU sed specific options with portable sed options.  */
-      for (i = 1; i < sub_argc; i++)
+      for (int i = 1; i < sub_argc; i++)
         {
           if (strcmp (sub_argv[i], "--expression") == 0)
             sub_argv[i] = "-e";
@@ -339,7 +343,7 @@ There is NO WARRANTY, to the extent permitted by law.\n\
     input_file = "-";
 
   /* Read input file.  */
-  result = read_catalog_file (input_file, input_syntax);
+  msgdomain_list_ty *result = read_catalog_file (input_file, input_syntax);
 
   /* Recognize special programs as built-ins.  */
   if (strcmp (sub_name, "recode-sr-latin") == 0 && sub_argc == 1)
@@ -347,21 +351,24 @@ There is NO WARRANTY, to the extent permitted by law.\n\
       filter = serbian_to_latin;
 
       /* Convert the input to UTF-8 first.  */
-      result = iconv_msgdomain_list (result, po_charset_utf8, true, input_file);
+      result = iconv_msgdomain_list (result, po_charset_utf8, true, input_file,
+                                     textmode_xerror_handler);
     }
   else if (strcmp (sub_name, "quot") == 0 && sub_argc == 1)
     {
       filter = ascii_quote_to_unicode;
 
       /* Convert the input to UTF-8 first.  */
-      result = iconv_msgdomain_list (result, po_charset_utf8, true, input_file);
+      result = iconv_msgdomain_list (result, po_charset_utf8, true, input_file,
+                                     textmode_xerror_handler);
     }
   else if (strcmp (sub_name, "boldquot") == 0 && sub_argc == 1)
     {
       filter = ascii_quote_to_unicode_bold;
 
       /* Convert the input to UTF-8 first.  */
-      result = iconv_msgdomain_list (result, po_charset_utf8, true, input_file);
+      result = iconv_msgdomain_list (result, po_charset_utf8, true, input_file,
+                                     textmode_xerror_handler);
     }
   else
     {
@@ -389,7 +396,8 @@ There is NO WARRANTY, to the extent permitted by law.\n\
     msgdomain_list_sort_by_msgid (result);
 
   /* Write the merged message list out.  */
-  msgdomain_list_print (result, output_file, output_syntax, force_po, false);
+  msgdomain_list_print (result, output_file, output_syntax,
+                        textmode_xerror_handler, force_po, false);
 
   exit (EXIT_SUCCESS);
 }
@@ -438,6 +446,12 @@ and writes a modified translation to standard output.\n\
 "));
       printf ("\n");
       printf (_("\
+Filter input and output:\n"));
+      printf (_("\
+  --newline                   add a newline at the end of input and\n\
+                                remove a newline from the end of output"));
+      printf ("\n");
+      printf (_("\
 Useful FILTER-OPTIONs when the FILTER is 'sed':\n"));
       printf (_("\
   -e, --expression=SCRIPT     add SCRIPT to the commands to be executed\n"));
@@ -499,12 +513,16 @@ Informative output:\n"));
       printf (_("\
   -V, --version               output version information and exit\n"));
       printf ("\n");
-      /* TRANSLATORS: The placeholder indicates the bug-reporting address
-         for this package.  Please add _another line_ saying
+      /* TRANSLATORS: The first placeholder is the web address of the Savannah
+         project of this package.  The second placeholder is the bug-reporting
+         email address for this package.  Please add _another line_ saying
          "Report translation bugs to <...>\n" with the address for translation
          bugs (typically your translation team's web or email address).  */
-      fputs (_("Report bugs to <bug-gnu-gettext@gnu.org>.\n"),
-             stdout);
+      printf (_("\
+Report bugs in the bug tracker at <%s>\n\
+or by email to <%s>.\n"),
+             "https://savannah.gnu.org/projects/gettext",
+             "bug-gettext@gnu.org");
     }
 
   exit (status);
@@ -554,7 +572,7 @@ prepare_read (size_t *num_bytes_p, void *private_data)
 
   if (l->length == l->allocated)
     {
-      l->allocated = l->allocated + (l->allocated >> 1);
+      l->allocated = l->allocated + (l->allocated >> 1) + 1;
       l->result = (char *) xrealloc (l->result, l->allocated);
     }
   *num_bytes_p = l->allocated - l->length;
@@ -577,7 +595,6 @@ static void
 generic_filter (const char *str, size_t len, char **resultp, size_t *lengthp)
 {
   struct locals l;
-
   l.str = str;
   l.len = len;
   l.allocated = len + (len >> 2) + 1;
@@ -601,7 +618,6 @@ process_string (const char *str, size_t len, char **resultp, size_t *lengthp)
 {
   char *result;
   size_t length;
-
   filter (str, len, &result, &length);
 
   /* Remove NUL bytes from result.  */
@@ -612,9 +628,7 @@ process_string (const char *str, size_t len, char **resultp, size_t *lengthp)
     for (; p < pend; p++)
       if (*p == '\0')
         {
-          char *q;
-
-          q = p;
+          char *q = p;
           for (; p < pend; p++)
             if (*p != '\0')
               *q++ = *p;
@@ -628,24 +642,43 @@ process_string (const char *str, size_t len, char **resultp, size_t *lengthp)
 }
 
 
+/* Do the same thing as process_string but append a newline to STR
+   before processing, and remove a newline from the result.
+ */
+static void
+process_string_with_newline (const char *str, size_t len, char **resultp,
+                             size_t *lengthp)
+{
+  char *newstr = XNMALLOC (len + 1, char);
+  memcpy (newstr, str, len);
+  newstr[len] = '\n';
+
+  char *result;
+  size_t length;
+  process_string (newstr, len + 1, &result, &length);
+
+  free (newstr);
+
+  if (length > 0 && result[length - 1] == '\n')
+    result[--length] = '\0';
+  else
+    error (0, 0, _("filter output is not terminated with a newline"));
+
+  *resultp = result;
+  *lengthp = length;
+}
+
+
 static void
 process_message (message_ty *mp)
 {
-  const char *msgstr = mp->msgstr;
-  size_t msgstr_len = mp->msgstr_len;
-  char *location;
-  size_t nsubstrings;
-  char **substrings;
-  size_t total_len;
-  char *total_str;
-  const char *p;
-  char *q;
-  size_t k;
-
   /* Keep the header entry unmodified, if --keep-header was given.  */
   if (is_header (mp) && keep_header)
     return;
 
+  const char *msgstr = mp->msgstr;
+  size_t msgstr_len = mp->msgstr_len;
+
   /* Set environment variables for the subprocess.
      Note: These environment variables, especially MSGFILTER_MSGCTXT and
      MSGFILTER_MSGID, may contain non-ASCII characters.  The subprocess
@@ -667,10 +700,12 @@ process_message (message_ty *mp)
     xsetenv ("MSGFILTER_MSGID_PLURAL", mp->msgid_plural, 1);
   else
     unsetenv ("MSGFILTER_MSGID_PLURAL");
-  location = xasprintf ("%s:%ld", mp->pos.file_name,
-                        (long) mp->pos.line_number);
-  xsetenv ("MSGFILTER_LOCATION", location, 1);
-  free (location);
+  {
+    char *location =
+      xasprintf ("%s:%ld", mp->pos.file_name, (long) mp->pos.line_number);
+    xsetenv ("MSGFILTER_LOCATION", location, 1);
+    free (location);
+  }
   if (mp->prev_msgctxt != NULL)
     xsetenv ("MSGFILTER_PREV_MSGCTXT", mp->prev_msgctxt, 1);
   else
@@ -685,45 +720,61 @@ process_message (message_ty *mp)
     unsetenv ("MSGFILTER_PREV_MSGID_PLURAL");
 
   /* Count NUL delimited substrings.  */
-  for (p = msgstr, nsubstrings = 0;
-       p < msgstr + msgstr_len;
-       p += strlen (p) + 1, nsubstrings++);
+  size_t nsubstrings;
+  {
+    nsubstrings = 0;
+    for (const char *p = msgstr; p < msgstr + msgstr_len; p += strlen (p) + 1)
+      nsubstrings++;
+  }
 
   /* Process each NUL delimited substring separately.  */
-  substrings = XNMALLOC (nsubstrings, char *);
-  for (p = msgstr, k = 0, total_len = 0; k < nsubstrings; k++)
-    {
-      char *result;
-      size_t length;
+  char **substrings = XNMALLOC (nsubstrings, char *);
+  size_t total_len;
+  {
+    const char *p;
+    size_t k;
+    for (p = msgstr, k = 0, total_len = 0; k < nsubstrings; k++)
+      {
+        if (mp->msgid_plural != NULL)
+          {
+            char *plural_form_string = xasprintf ("%lu", (unsigned long) k);
 
-      if (mp->msgid_plural != NULL)
-        {
-          char *plural_form_string = xasprintf ("%lu", k);
+            xsetenv ("MSGFILTER_PLURAL_FORM", plural_form_string, 1);
+            free (plural_form_string);
+          }
+        else
+          unsetenv ("MSGFILTER_PLURAL_FORM");
 
-          xsetenv ("MSGFILTER_PLURAL_FORM", plural_form_string, 1);
-          free (plural_form_string);
-        }
-      else
-        unsetenv ("MSGFILTER_PLURAL_FORM");
-      process_string (p, strlen (p), &result, &length);
-      result = (char *) xrealloc (result, length + 1);
-      result[length] = '\0';
-      substrings[k] = result;
-      total_len += length + 1;
+        char *result;
+        size_t length;
+        if (newline)
+          process_string_with_newline (p, strlen (p), &result, &length);
+        else
+          process_string (p, strlen (p), &result, &length);
 
-      p += strlen (p) + 1;
-    }
+        result = (char *) xrealloc (result, length + 1);
+        result[length] = '\0';
+        substrings[k] = result;
+        total_len += length + 1;
+
+        p += strlen (p) + 1;
+      }
+  }
 
   /* Concatenate the results, including the NUL after each.  */
-  total_str = XNMALLOC (total_len, char);
-  for (k = 0, q = total_str; k < nsubstrings; k++)
-    {
-      size_t length = strlen (substrings[k]);
+  char *total_str = XNMALLOC (total_len, char);
+  {
+    size_t k;
+    char *q;
+    for (k = 0, q = total_str; k < nsubstrings; k++)
+      {
+        size_t length = strlen (substrings[k]);
 
-      memcpy (q, substrings[k], length + 1);
-      free (substrings[k]);
-      q += length + 1;
-    }
+        memcpy (q, substrings[k], length + 1);
+        free (substrings[k]);
+        q += length + 1;
+      }
+  }
   free (substrings);
 
   mp->msgstr = total_str;
@@ -734,9 +785,7 @@ process_message (message_ty *mp)
 static void
 process_message_list (message_list_ty *mlp)
 {
-  size_t j;
-
-  for (j = 0; j < mlp->nitems; j++)
+  for (size_t j = 0; j < mlp->nitems; j++)
     process_message (mlp->item[j]);
 }
 
@@ -744,9 +793,7 @@ process_message_list (message_list_ty *mlp)
 static msgdomain_list_ty *
 process_msgdomain_list (msgdomain_list_ty *mdlp)
 {
-  size_t k;
-
-  for (k = 0; k < mdlp->nitems; k++)
+  for (size_t k = 0; k < mdlp->nitems; k++)
     process_message_list (mdlp->item[k]->messages);
 
   return mdlp;
