@@ -1,6 +1,5 @@
 /* Message list charset and locale charset handling.
-   Copyright (C) 2001-2003 Free Software Foundation, Inc.
-   Written by Bruno Haible <haible@clisp.cons.org>, 2001.
+   Copyright (C) 2001-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,6 +14,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
+/* Written by Bruno Haible.  */
+
 #ifndef _MSGL_CHARSET_H
 #define _MSGL_CHARSET_H
 
@@ -26,6 +27,9 @@ extern "C" {
 #endif
 
 
+extern void
+       check_pot_charset (const msgdomain_list_ty *mdlp, const char *filename);
+
 extern void
        compare_po_locale_charsets (const msgdomain_list_ty *mdlp);
 
