@@ -1,6 +1,5 @@
 /* Reading binary .mo files.
-   Copyright (C) 1995-1998, 2000-2007, 2014-2015, 2017 Free Software Foundation, Inc.
-   Written by Ulrich Drepper <drepper@gnu.ai.mit.edu>, April 1995.
+   Copyright (C) 1995-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,9 +14,9 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+/* Written by Ulrich Drepper and Bruno Haible.  */
+
+#include <config.h>
 
 /* Specification.  */
 #include "read-mo.h"
@@ -29,10 +28,11 @@
 #include <stdlib.h>
 #include <string.h>
 
-/* This include file describes the main part of binary .mo format.  */
+/* These two include files describe the binary .mo format.  */
 #include "gmo.h"
+#include "hash-string.h"
 
-#include "error.h"
+#include <error.h>
 #include "xalloc.h"
 #include "binary-io.h"
 #include "message.h"
@@ -68,8 +68,6 @@ read_binary_mo_file (struct binary_mo_file *bfp,
   char *buf = NULL;
   size_t alloc = 0;
   size_t size = 0;
-  size_t count;
-
   while (!feof (fp))
     {
       const size_t increment = 4096;
@@ -80,7 +78,7 @@ read_binary_mo_file (struct binary_mo_file *bfp,
             alloc = size + increment;
           buf = (char *) xrealloc (buf, alloc);
         }
-      count = fread (buf + size, 1, increment, fp);
+      size_t count = fread (buf + size, 1, increment, fp);
       if (count == 0)
         {
           if (ferror (fp))
@@ -91,6 +89,7 @@ read_binary_mo_file (struct binary_mo_file *bfp,
         size += count;
     }
   buf = (char *) xrealloc (buf, size);
+
   bfp->filename = filename;
   bfp->data = buf;
   bfp->size = size;
@@ -100,16 +99,14 @@ read_binary_mo_file (struct binary_mo_file *bfp,
 static nls_uint32
 get_uint32 (const struct binary_mo_file *bfp, size_t offset)
 {
-  nls_uint32 b0, b1, b2, b3;
   size_t end = xsum (offset, 4);
-
   if (size_overflow_p (end) || end > bfp->size)
     error (EXIT_FAILURE, 0, _("file \"%s\" is truncated"), bfp->filename);
 
-  b0 = *(unsigned char *) (bfp->data + offset + 0);
-  b1 = *(unsigned char *) (bfp->data + offset + 1);
-  b2 = *(unsigned char *) (bfp->data + offset + 2);
-  b3 = *(unsigned char *) (bfp->data + offset + 3);
+  nls_uint32 b0 = *(unsigned char *) (bfp->data + offset + 0);
+  nls_uint32 b1 = *(unsigned char *) (bfp->data + offset + 1);
+  nls_uint32 b2 = *(unsigned char *) (bfp->data + offset + 2);
+  nls_uint32 b3 = *(unsigned char *) (bfp->data + offset + 3);
   if (bfp->endian == MO_LITTLE_ENDIAN)
     return b0 | (b1 << 8) | (b2 << 16) | (b3 << 24);
   else
@@ -123,10 +120,11 @@ get_string (const struct binary_mo_file *bfp, size_t offset, size_t *lengthp)
   /* See 'struct string_desc'.  */
   nls_uint32 s_length = get_uint32 (bfp, offset);
   nls_uint32 s_offset = get_uint32 (bfp, offset + 4);
-  size_t s_end = xsum3 (s_offset, s_length, 1);
 
+  size_t s_end = xsum3 (s_offset, s_length, 1);
   if (size_overflow_p (s_end) || s_end > bfp->size)
     error (EXIT_FAILURE, 0, _("file \"%s\" is truncated"), bfp->filename);
+
   if (bfp->data[s_offset + s_length] != '\0')
     error (EXIT_FAILURE, 0,
            _("file \"%s\" contains a not NUL terminated string"),
@@ -142,27 +140,16 @@ get_sysdep_string (const struct binary_mo_file *bfp, size_t offset,
                    const struct mo_file_header *header, size_t *lengthp)
 {
   /* See 'struct sysdep_string'.  */
-  size_t length;
-  char *string;
-  size_t i;
-  char *p;
-  nls_uint32 s_offset;
 
   /* Compute the length.  */
-  s_offset = get_uint32 (bfp, offset);
-  length = 0;
-  for (i = 4; ; i += 8)
+  nls_uint32 s_offset = get_uint32 (bfp, offset);
+  size_t length = 0;
+  for (size_t i = 4; ; i += 8)
     {
       nls_uint32 segsize = get_uint32 (bfp, offset + i);
       nls_uint32 sysdepref = get_uint32 (bfp, offset + i + 4);
-      nls_uint32 sysdep_segment_offset;
-      nls_uint32 ss_length;
-      nls_uint32 ss_offset;
-      size_t ss_end;
-      size_t s_end;
-      size_t n;
 
-      s_end = xsum (s_offset, segsize);
+      size_t s_end = xsum (s_offset, segsize);
       if (size_overflow_p (s_end) || s_end > bfp->size)
         error (EXIT_FAILURE, 0, _("file \"%s\" is truncated"), bfp->filename);
       length += segsize;
@@ -183,10 +170,10 @@ get_sysdep_string (const struct binary_mo_file *bfp, size_t offset,
         error (EXIT_FAILURE, 0, _("file \"%s\" is not in GNU .mo format"),
                bfp->filename);
       /* See 'struct sysdep_segment'.  */
-      sysdep_segment_offset = header->sysdep_segments_offset + sysdepref * 8;
-      ss_length = get_uint32 (bfp, sysdep_segment_offset);
-      ss_offset = get_uint32 (bfp, sysdep_segment_offset + 4);
-      ss_end = xsum (ss_offset, ss_length);
+      nls_uint32 sysdep_segment_offset = header->sysdep_segments_offset + sysdepref * 8;
+      nls_uint32 ss_length = get_uint32 (bfp, sysdep_segment_offset);
+      nls_uint32 ss_offset = get_uint32 (bfp, sysdep_segment_offset + 4);
+      size_t ss_end = xsum (ss_offset, ss_length);
       if (size_overflow_p (ss_end) || ss_end > bfp->size)
         error (EXIT_FAILURE, 0, _("file \"%s\" is truncated"), bfp->filename);
       if (!(ss_length > 0 && bfp->data[ss_end - 1] == '\0'))
@@ -197,50 +184,48 @@ get_sysdep_string (const struct binary_mo_file *bfp, size_t offset,
                  _("file \"%s\" contains a not NUL terminated string, at %s"),
                  bfp->filename, location);
         }
-      n = strlen (bfp->data + ss_offset);
+      size_t n = strlen (bfp->data + ss_offset);
       length += (n > 1 ? 1 + n + 1 : n);
     }
 
   /* Allocate and fill the string.  */
-  string = XNMALLOC (length, char);
-  p = string;
-  s_offset = get_uint32 (bfp, offset);
-  for (i = 4; ; i += 8)
-    {
-      nls_uint32 segsize = get_uint32 (bfp, offset + i);
-      nls_uint32 sysdepref = get_uint32 (bfp, offset + i + 4);
-      nls_uint32 sysdep_segment_offset;
-      nls_uint32 ss_length;
-      nls_uint32 ss_offset;
-      size_t n;
+  char *string = XNMALLOC (length, char);
+  {
+    char *p = string;
+    s_offset = get_uint32 (bfp, offset);
+    for (size_t i = 4; ; i += 8)
+      {
+        nls_uint32 segsize = get_uint32 (bfp, offset + i);
+        nls_uint32 sysdepref = get_uint32 (bfp, offset + i + 4);
 
-      memcpy (p, bfp->data + s_offset, segsize);
-      p += segsize;
-      s_offset += segsize;
+        memcpy (p, bfp->data + s_offset, segsize);
+        p += segsize;
+        s_offset += segsize;
 
-      if (sysdepref == SEGMENTS_END)
-        break;
-      if (sysdepref >= header->n_sysdep_segments)
-        abort ();
-      /* See 'struct sysdep_segment'.  */
-      sysdep_segment_offset = header->sysdep_segments_offset + sysdepref * 8;
-      ss_length = get_uint32 (bfp, sysdep_segment_offset);
-      ss_offset = get_uint32 (bfp, sysdep_segment_offset + 4);
-      if (ss_offset + ss_length > bfp->size)
-        abort ();
-      if (!(ss_length > 0 && bfp->data[ss_offset + ss_length - 1] == '\0'))
-        abort ();
-      n = strlen (bfp->data + ss_offset);
-      if (n > 1)
-        *p++ = '<';
-      memcpy (p, bfp->data + ss_offset, n);
-      p += n;
-      if (n > 1)
-        *p++ = '>';
-    }
+        if (sysdepref == SEGMENTS_END)
+          break;
+        if (sysdepref >= header->n_sysdep_segments)
+          abort ();
+        /* See 'struct sysdep_segment'.  */
+        nls_uint32 sysdep_segment_offset = header->sysdep_segments_offset + sysdepref * 8;
+        nls_uint32 ss_length = get_uint32 (bfp, sysdep_segment_offset);
+        nls_uint32 ss_offset = get_uint32 (bfp, sysdep_segment_offset + 4);
+        if (ss_offset + ss_length > bfp->size)
+          abort ();
+        if (!(ss_length > 0 && bfp->data[ss_offset + ss_length - 1] == '\0'))
+          abort ();
+        size_t n = strlen (bfp->data + ss_offset);
+        if (n > 1)
+          *p++ = '<';
+        memcpy (p, bfp->data + ss_offset, n);
+        p += n;
+        if (n > 1)
+          *p++ = '>';
+      }
 
-  if (p != string + length)
-    abort ();
+    if (p != string + length)
+      abort ();
+  }
 
   *lengthp = length;
   return string;
@@ -250,12 +235,9 @@ get_sysdep_string (const struct binary_mo_file *bfp, size_t offset,
 void
 read_mo_file (message_list_ty *mlp, const char *filename)
 {
-  FILE *fp;
-  struct binary_mo_file bf;
-  struct mo_file_header header;
-  unsigned int i;
   static lex_pos_ty pos = { __FILE__, __LINE__ };
 
+  FILE *fp;
   if (strcmp (filename, "-") == 0 || strcmp (filename, "/dev/stdin") == 0)
     {
       fp = stdin;
@@ -270,12 +252,15 @@ read_mo_file (message_list_ty *mlp, const char *filename)
     }
 
   /* Read the file contents into memory.  */
+  struct binary_mo_file bf;
   read_binary_mo_file (&bf, fp, filename);
 
   /* Get a 32-bit number from the file header.  */
 # define GET_HEADER_FIELD(field) \
     get_uint32 (&bf, offsetof (struct mo_file_header, field))
 
+  struct mo_file_header header;
+
   /* We must grope the file to determine which endian it is.
      Perversity of the universe tends towards maximum, so it will
      probably not match the currently executing architecture.  */
@@ -307,43 +292,127 @@ read_mo_file (message_list_ty *mlp, const char *filename)
       header.hash_tab_size = GET_HEADER_FIELD (hash_tab_size);
       header.hash_tab_offset = GET_HEADER_FIELD (hash_tab_offset);
 
-      for (i = 0; i < header.nstrings; i++)
-        {
-          message_ty *mp;
-          char *msgctxt;
-          char *msgid;
-          size_t msgid_len;
-          char *separator;
-          char *msgstr;
-          size_t msgstr_len;
+      /* The following verifications attempt to ensure that 'msgunfmt' complains
+         about a .mo file that may make libintl crash at run time.  */
 
-          /* Read the msgctxt and msgid.  */
-          msgid = get_string (&bf, header.orig_tab_offset + i * 8,
-                              &msgid_len);
-          /* Split into msgctxt and msgid.  */
-          separator = strchr (msgid, MSGCTXT_SEPARATOR);
-          if (separator != NULL)
+      /* Verify that the array of messages is sorted.  */
+      {
+        char *prev_msgid = NULL;
+
+        for (unsigned int i = 0; i < header.nstrings; i++)
+          {
+            size_t msgid_len;
+            char *msgid = get_string (&bf, header.orig_tab_offset + i * 8,
+                                      &msgid_len);
+            if (i == 0)
+              prev_msgid = msgid;
+            else
+              {
+                if (!(strcmp (prev_msgid, msgid) < 0))
+                  error (EXIT_FAILURE, 0,
+                         _("file \"%s\" is not in GNU .mo format: The array of messages is not sorted."),
+                         filename);
+              }
+          }
+      }
+
+      /* Verify the hash table.  */
+      if (header.hash_tab_size > 0)
+        {
+          /* Verify the hash table's size.  */
+          if (!(header.hash_tab_size > 2))
+            error (EXIT_FAILURE, 0,
+                   _("file \"%s\" is not in GNU .mo format: The hash table size is invalid."),
+                   filename);
+
+          /* Verify that the non-empty hash table entries contain the values
+             1, ..., nstrings, each exactly once.  */
+          char *seen = (char *) xcalloc (header.nstrings, 1);
+          for (unsigned int j = 0; j < header.hash_tab_size; j++)
             {
-              /* The part before the MSGCTXT_SEPARATOR is the msgctxt.  */
-              *separator = '\0';
-              msgctxt = msgid;
-              msgid = separator + 1;
-              msgid_len -= msgid - msgctxt;
+              nls_uint32 entry =
+                get_uint32 (&bf, header.hash_tab_offset + j * 4);
+
+              if (entry != 0)
+                {
+                  unsigned int i = entry - 1;
+                  if (!(i < header.nstrings && seen[i] == 0))
+                    error (EXIT_FAILURE, 0,
+                           _("file \"%s\" is not in GNU .mo format: The hash table contains invalid entries."),
+                           filename);
+                  seen[i] = 1;
+                }
             }
-          else
-            msgctxt = NULL;
+          for (unsigned int i = 0; i < header.nstrings; i++)
+            if (seen[i] == 0)
+              error (EXIT_FAILURE, 0, _("file \"%s\" is not in GNU .mo format: Some messages are not present in the hash table."),
+                     filename);
+          free (seen);
+
+          /* Verify that the hash table lookup algorithm finds the entry for
+             each message.  */
+          for (unsigned int i = 0; i < header.nstrings; i++)
+            {
+              size_t msgid_len;
+              char *msgid = get_string (&bf, header.orig_tab_offset + i * 8,
+                                        &msgid_len);
+              nls_uint32 hash_val = hash_string (msgid);
+              nls_uint32 idx = hash_val % header.hash_tab_size;
+              nls_uint32 incr = 1 + (hash_val % (header.hash_tab_size - 2));
+              for (;;)
+                {
+                  nls_uint32 entry =
+                    get_uint32 (&bf, header.hash_tab_offset + idx * 4);
+
+                  if (entry == 0)
+                    error (EXIT_FAILURE, 0,
+                           _("file \"%s\" is not in GNU .mo format: Some messages are at a wrong index in the hash table."),
+                           filename);
+                  if (entry == i + 1)
+                    break;
+
+                  if (idx >= header.hash_tab_size - incr)
+                    idx -= header.hash_tab_size - incr;
+                  else
+                    idx += incr;
+                }
+            }
+        }
+
+      for (unsigned int i = 0; i < header.nstrings; i++)
+        {
+          /* Read the msgctxt and msgid.  */
+          size_t msgid_len;
+          char *msgid = get_string (&bf, header.orig_tab_offset + i * 8,
+                                    &msgid_len);
+          /* Split into msgctxt and msgid.  */
+          char *msgctxt;
+          {
+            char *separator = strchr (msgid, MSGCTXT_SEPARATOR);
+            if (separator != NULL)
+              {
+                /* The part before the MSGCTXT_SEPARATOR is the msgctxt.  */
+                *separator = '\0';
+                msgctxt = msgid;
+                msgid = separator + 1;
+                msgid_len -= msgid - msgctxt;
+              }
+            else
+              msgctxt = NULL;
+          }
 
           /* Read the msgstr.  */
-          msgstr = get_string (&bf, header.trans_tab_offset + i * 8,
-                               &msgstr_len);
+          size_t msgstr_len;
+          char *msgstr = get_string (&bf, header.trans_tab_offset + i * 8,
+                                     &msgstr_len);
 
-          mp = message_alloc (msgctxt,
-                              msgid,
-                              (strlen (msgid) + 1 < msgid_len
-                               ? msgid + strlen (msgid) + 1
-                               : NULL),
-                              msgstr, msgstr_len,
-                              &pos);
+          message_ty *mp = message_alloc (msgctxt,
+                                          msgid,
+                                          (strlen (msgid) + 1 < msgid_len
+                                           ? msgid + strlen (msgid) + 1
+                                           : NULL),
+                                          msgstr, msgstr_len,
+                                          &pos);
           message_list_append (mlp, mp);
         }
 
@@ -363,78 +432,80 @@ read_mo_file (message_list_ty *mlp, const char *filename)
           header.trans_sysdep_tab_offset =
             GET_HEADER_FIELD (trans_sysdep_tab_offset);
 
-          for (i = 0; i < header.n_sysdep_strings; i++)
+          for (unsigned int i = 0; i < header.n_sysdep_strings; i++)
             {
-              message_ty *mp;
-              char *msgctxt;
+              /* Read the msgctxt and msgid.  */
               char *msgid;
               size_t msgid_len;
-              char *separator;
-              char *msgstr;
-              size_t msgstr_len;
-              nls_uint32 offset;
-              size_t f;
-
-              /* Read the msgctxt and msgid.  */
-              offset = get_uint32 (&bf, header.orig_sysdep_tab_offset + i * 4);
-              msgid = get_sysdep_string (&bf, offset, &header, &msgid_len);
+              {
+                nls_uint32 offset = get_uint32 (&bf, header.orig_sysdep_tab_offset + i * 4);
+                msgid = get_sysdep_string (&bf, offset, &header, &msgid_len);
+              }
               /* Split into msgctxt and msgid.  */
-              separator = strchr (msgid, MSGCTXT_SEPARATOR);
-              if (separator != NULL)
-                {
-                  /* The part before the MSGCTXT_SEPARATOR is the msgctxt.  */
-                  *separator = '\0';
-                  msgctxt = msgid;
-                  msgid = separator + 1;
-                  msgid_len -= msgid - msgctxt;
-                }
-              else
-                msgctxt = NULL;
+              char *msgctxt;
+              {
+                char *separator = strchr (msgid, MSGCTXT_SEPARATOR);
+                if (separator != NULL)
+                  {
+                    /* The part before the MSGCTXT_SEPARATOR is the msgctxt.  */
+                    *separator = '\0';
+                    msgctxt = msgid;
+                    msgid = separator + 1;
+                    msgid_len -= msgid - msgctxt;
+                  }
+                else
+                  msgctxt = NULL;
+              }
 
               /* Read the msgstr.  */
-              offset = get_uint32 (&bf, header.trans_sysdep_tab_offset + i * 4);
-              msgstr = get_sysdep_string (&bf, offset, &header, &msgstr_len);
+              char *msgstr;
+              size_t msgstr_len;
+              {
+                nls_uint32 offset = get_uint32 (&bf, header.trans_sysdep_tab_offset + i * 4);
+                msgstr = get_sysdep_string (&bf, offset, &header, &msgstr_len);
+              }
 
-              mp = message_alloc (msgctxt,
-                                  msgid,
-                                  (strlen (msgid) + 1 < msgid_len
-                                   ? msgid + strlen (msgid) + 1
-                                   : NULL),
-                                  msgstr, msgstr_len,
-                                  &pos);
+              message_ty *mp = message_alloc (msgctxt,
+                                              msgid,
+                                              (strlen (msgid) + 1 < msgid_len
+                                               ? msgid + strlen (msgid) + 1
+                                               : NULL),
+                                              msgstr, msgstr_len,
+                                              &pos);
 
               /* Only messages with c-format or objc-format annotation are
                  recognized as having system-dependent strings by msgfmt.
                  Which one of the two, we don't know.  We have to guess,
                  assuming that c-format is more probable than objc-format and
                  that the .mo was likely produced by "msgfmt -c".  */
-              for (f = format_c; ; f = format_objc)
+              for (size_t f = format_c; ; f = format_objc)
                 {
-                  bool valid = true;
                   struct formatstring_parser *parser = formatstring_parsers[f];
-                  const char *str_end;
-                  const char *str;
 
-                  str_end = msgid + msgid_len;
-                  for (str = msgid; str < str_end; str += strlen (str) + 1)
-                    {
-                      char *invalid_reason = NULL;
-                      void *descr =
-                        parser->parse (str, false, NULL, &invalid_reason);
+                  bool valid = true;
+                  {
+                    const char *str_end = msgid + msgid_len;
+                    for (const char *str = msgid; str < str_end; str += strlen (str) + 1)
+                      {
+                        char *invalid_reason = NULL;
+                        void *descr =
+                          parser->parse (str, false, NULL, &invalid_reason);
+
+                        if (descr != NULL)
+                          parser->free (descr);
+                        else
+                          {
+                            free (invalid_reason);
+                            valid = false;
+                            break;
+                          }
+                      }
+                  }
 
-                      if (descr != NULL)
-                        parser->free (descr);
-                      else
-                        {
-                          free (invalid_reason);
-                          valid = false;
-                          break;
-                        }
-                    }
                   if (valid)
                     {
-                      str_end = msgstr + msgstr_len;
-                      for (str = msgstr; str < str_end; str += strlen (str) + 1)
+                      const char *str_end = msgstr + msgstr_len;
+                      for (const char *str = msgstr; str < str_end; str += strlen (str) + 1)
                         {
                           char *invalid_reason = NULL;
                           void *descr =
