@@ -1,6 +1,5 @@
 /* Reading PO files.
-   Copyright (C) 2006-2024 Free Software Foundation, Inc.
-   Written by Bruno Haible <bruno@clisp.org>, 2006.
+   Copyright (C) 2006-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,6 +14,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
+/* Written by Bruno Haible.  */
+
 #ifndef _READ_PO_H
 #define _READ_PO_H
 
@@ -27,14 +28,14 @@ extern "C" {
 
 
 /* Describes a .po / .pot file parser.  */
-extern DLL_VARIABLE const struct catalog_input_format input_format_po;
+extern LIBGETTEXTSRC_DLL_VARIABLE const struct catalog_input_format input_format_po;
 
 
 /* Global variables.  */
 
 /* Number of parse errors within a PO file that cause the program to
    terminate.  Cf. error_message_count, declared in <error.h>.  */
-extern DLL_VARIABLE unsigned int gram_max_allowed_errors;
+extern LIBGETTEXTSRC_DLL_VARIABLE unsigned int gram_max_allowed_errors;
 
 
 #ifdef __cplusplus
