@@ -1,6 +1,5 @@
 /* Reading Java .properties files.
-   Copyright (C) 2003, 2006 Free Software Foundation, Inc.
-   Written by Bruno Haible <bruno@clisp.org>, 2003.
+   Copyright (C) 2003-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,12 +14,26 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
+/* Written by Bruno Haible.  */
+
 #ifndef _READ_PROPERTIES_H
 #define _READ_PROPERTIES_H
 
 #include "read-catalog-abstract.h"
 
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+
 /* Describes a .properties file parser.  */
-extern DLL_VARIABLE const struct catalog_input_format input_format_properties;
+extern LIBGETTEXTSRC_DLL_VARIABLE const struct catalog_input_format input_format_properties;
+
+
+#ifdef __cplusplus
+}
+#endif
+
 
 #endif /* _READ_PROPERTIES_H */
