@@ -1,7 +1,5 @@
 /* Routines for locating data files
-   Copyright (C) 2016-2024 Free Software Foundation, Inc.
-
-   This file was written by Daiki Ueno <ueno@gnu.org>, 2016.
+   Copyright (C) 2016-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,9 +14,9 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+/* Written by Daiki Ueno and Bruno Haible.  */
+
+#include <config.h>
 
 /* Specification.  */
 #include "search-path.h"
@@ -88,10 +86,10 @@ static void
 fill (const char *dir, size_t len, void *data)
 {
   struct path_array_ty *array = data;
-  char *base;
-  char *name;
 
-  base = xmemdup0 (dir, len);
+  char *base = xmemdup0 (dir, len);
+
+  char *name;
   if (array->sub == NULL)
     name = base;
   else
@@ -123,29 +121,25 @@ fill (const char *dir, size_t len, void *data)
 char **
 get_search_path (const char *sub)
 {
-  const char *gettextdatadir;
-  const char *gettextdatadirs;
-  const char *xdgdatadirs;
-  struct path_array_ty array;
-
   /* Count how many array elements are needed.  */
   size_t count = 2;
 
-  gettextdatadirs = getenv ("GETTEXTDATADIRS");
+  const char *gettextdatadirs = getenv ("GETTEXTDATADIRS");
   if (gettextdatadirs != NULL)
     foreach_elements (gettextdatadirs, increment, &count);
 
-  xdgdatadirs = getenv ("XDG_DATA_DIRS");
+  const char *xdgdatadirs = getenv ("XDG_DATA_DIRS");
   if (xdgdatadirs != NULL)
     foreach_elements (xdgdatadirs, increment, &count);
 
   /* Allocate the array.  */
+  struct path_array_ty array;
   array.ptr = XNMALLOC (count + 1, char *);
   array.len = 0;
 
   /* Fill the array.  */
   {
-    gettextdatadir = getenv ("GETTEXTDATADIR");
+    const char *gettextdatadir = getenv ("GETTEXTDATADIR");
     if (gettextdatadir == NULL || gettextdatadir[0] == '\0')
       /* Make it possible to override the locator file location.  This
          is necessary for running the testsuite before "make
