@@ -1,8 +1,5 @@
 /* GNU gettext - internationalization aids
-   Copyright (C) 1995, 1998, 2000-2004, 2006, 2009 Free Software
-   Foundation, Inc.
-
-   This file was written by Peter Miller <millerp@canb.auug.org.au>
+   Copyright (C) 1995-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,9 +14,9 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
+/* Written by Peter Miller, Ulrich Drepper, and Bruno Haible.  */
+
+#include <config.h>
 
 /* Specification.  */
 #include "str-list.h"
@@ -45,9 +42,7 @@ string_list_init (string_list_ty *slp)
 string_list_ty *
 string_list_alloc ()
 {
-  string_list_ty *slp;
-
-  slp = XMALLOC (string_list_ty);
+  string_list_ty *slp = XMALLOC (string_list_ty);
   slp->item = NULL;
   slp->nitems = 0;
   slp->nitems_max = 0;
@@ -63,10 +58,8 @@ string_list_append (string_list_ty *slp, const char *s)
   /* Grow the list.  */
   if (slp->nitems >= slp->nitems_max)
     {
-      size_t nbytes;
-
       slp->nitems_max = slp->nitems_max * 2 + 4;
-      nbytes = slp->nitems_max * sizeof (slp->item[0]);
+      size_t nbytes = slp->nitems_max * sizeof (slp->item[0]);
       slp->item = (const char **) xrealloc (slp->item, nbytes);
     }
 
@@ -75,15 +68,31 @@ string_list_append (string_list_ty *slp, const char *s)
 }
 
 
+/* Append a freshly allocated single string to the end of a list of strings,
+   transferring its ownership to SLP.  */
+void
+string_list_append_move (string_list_ty *slp, char *s)
+{
+  /* Grow the list.  */
+  if (slp->nitems >= slp->nitems_max)
+    {
+      slp->nitems_max = slp->nitems_max * 2 + 4;
+      size_t nbytes = slp->nitems_max * sizeof (slp->item[0]);
+      slp->item = (const char **) xrealloc (slp->item, nbytes);
+    }
+
+  /* Add the string itself to the end of the list.  */
+  slp->item[slp->nitems++] = s;
+}
+
+
 /* Append a single string to the end of a list of strings, unless it is
    already contained in the list.  */
 void
 string_list_append_unique (string_list_ty *slp, const char *s)
 {
-  size_t j;
-
-  /* Do not if the string is already in the list.  */
-  for (j = 0; j < slp->nitems; ++j)
+  /* Do nothing if the string is already in the list.  */
+  for (size_t j = 0; j < slp->nitems; ++j)
     if (strcmp (slp->item[j], s) == 0)
       return;
 
@@ -100,14 +109,41 @@ string_list_append_unique (string_list_ty *slp, const char *s)
   slp->item[slp->nitems++] = xstrdup (s);
 }
 
+/* Likewise with a string descriptor as argument.  */
+void
+string_list_append_unique_desc (string_list_ty *slp,
+                                const char *s, size_t s_len)
+{
+  /* Do nothing if the string is already in the list.  */
+  for (size_t j = 0; j < slp->nitems; ++j)
+    if (strlen (slp->item[j]) == s_len && memcmp (slp->item[j], s, s_len) == 0)
+      return;
+
+  /* Grow the list.  */
+  if (slp->nitems >= slp->nitems_max)
+    {
+      slp->nitems_max = slp->nitems_max * 2 + 4;
+      slp->item = (const char **) xrealloc (slp->item,
+                                            slp->nitems_max
+                                            * sizeof (slp->item[0]));
+    }
+
+  /* Add a copy of the string to the end of the list.  */
+  {
+    char *copy = XNMALLOC (s_len + 1, char);
+    memcpy (copy, s, s_len);
+    copy[s_len] = '\0';
+
+    slp->item[slp->nitems++] = copy;
+  }
+}
+
 
 /* Destroy a list of strings.  */
 void
 string_list_destroy (string_list_ty *slp)
 {
-  size_t j;
-
-  for (j = 0; j < slp->nitems; ++j)
+  for (size_t j = 0; j < slp->nitems; ++j)
     free ((char *) slp->item[j]);
   if (slp->item != NULL)
     free (slp->item);
@@ -118,9 +154,7 @@ string_list_destroy (string_list_ty *slp)
 void
 string_list_free (string_list_ty *slp)
 {
-  size_t j;
-
-  for (j = 0; j < slp->nitems; ++j)
+  for (size_t j = 0; j < slp->nitems; ++j)
     free ((char *) slp->item[j]);
   if (slp->item != NULL)
     free (slp->item);
@@ -133,23 +167,23 @@ string_list_free (string_list_ty *slp)
 char *
 string_list_concat (const string_list_ty *slp)
 {
-  size_t len;
-  size_t j;
   char *result;
-  size_t pos;
-
-  len = 1;
-  for (j = 0; j < slp->nitems; ++j)
-    len += strlen (slp->item[j]);
-  result = XNMALLOC (len, char);
-  pos = 0;
-  for (j = 0; j < slp->nitems; ++j)
-    {
-      len = strlen (slp->item[j]);
-      memcpy (result + pos, slp->item[j], len);
-      pos += len;
-    }
-  result[pos] = '\0';
+  {
+    size_t len = 1;
+    for (size_t j = 0; j < slp->nitems; ++j)
+      len += strlen (slp->item[j]);
+    result = XNMALLOC (len, char);
+  }
+  {
+    size_t pos = 0;
+    for (size_t j = 0; j < slp->nitems; ++j)
+      {
+        size_t len = strlen (slp->item[j]);
+        memcpy (result + pos, slp->item[j], len);
+        pos += len;
+      }
+    result[pos] = '\0';
+  }
   return result;
 }
 
@@ -186,40 +220,44 @@ string_list_join (const string_list_ty *slp, const char *separator,
                   char terminator, bool drop_redundant_terminator)
 {
   size_t separator_len = strlen (separator);
-  size_t len;
-  size_t j;
-  char *result;
-  size_t pos;
 
-  len = 1;
-  for (j = 0; j < slp->nitems; ++j)
+  char *result;
+  {
+    size_t len = 1;
+    for (size_t j = 0; j < slp->nitems; ++j)
+      {
+        if (j > 0)
+          len += separator_len;
+        len += strlen (slp->item[j]);
+      }
+    if (terminator)
+      ++len;
+    result = XNMALLOC (len, char);
+  }
+  {
+    size_t pos = 0;
+    for (size_t j = 0; j < slp->nitems; ++j)
+      {
+        if (j > 0)
+          {
+            memcpy (result + pos, separator, separator_len);
+            pos += separator_len;
+          }
+        size_t len = strlen (slp->item[j]);
+        memcpy (result + pos, slp->item[j], len);
+        pos += len;
+      }
     {
-      if (j > 0)
-        len += separator_len;
-      len += strlen (slp->item[j]);
+      size_t len;
+      if (terminator
+          && !(drop_redundant_terminator
+               && slp->nitems > 0
+               && (len = strlen (slp->item[slp->nitems - 1])) > 0
+               && slp->item[slp->nitems - 1][len - 1] == terminator))
+        result[pos++] = terminator;
     }
-  if (terminator)
-    ++len;
-  result = XNMALLOC (len, char);
-  pos = 0;
-  for (j = 0; j < slp->nitems; ++j)
-    {
-      if (j > 0)
-        {
-          memcpy (result + pos, separator, separator_len);
-          pos += separator_len;
-        }
-      len = strlen (slp->item[j]);
-      memcpy (result + pos, slp->item[j], len);
-      pos += len;
-    }
-  if (terminator
-      && !(drop_redundant_terminator
-           && slp->nitems > 0
-           && (len = strlen (slp->item[slp->nitems - 1])) > 0
-           && slp->item[slp->nitems - 1][len - 1] == terminator))
-    result[pos++] = terminator;
-  result[pos] = '\0';
+    result[pos] = '\0';
+  }
   return result;
 }
 
@@ -228,10 +266,36 @@ string_list_join (const string_list_ty *slp, const char *separator,
 bool
 string_list_member (const string_list_ty *slp, const char *s)
 {
-  size_t j;
-
-  for (j = 0; j < slp->nitems; ++j)
+  for (size_t j = 0; j < slp->nitems; ++j)
     if (strcmp (slp->item[j], s) == 0)
       return true;
   return false;
 }
+
+/* Likewise with a string descriptor as argument.  */
+bool
+string_list_member_desc (const string_list_ty *slp, const char *s, size_t s_len)
+{
+  for (size_t j = 0; j < slp->nitems; ++j)
+    if (strlen (slp->item[j]) == s_len && memcmp (slp->item[j], s, s_len) == 0)
+      return true;
+  return false;
+}
+
+
+/* Remove s from the list of strings.  Return the removed string or NULL.  */
+const char *
+string_list_remove (string_list_ty *slp, const char *s)
+{
+  for (size_t j = 0; j < slp->nitems; ++j)
+    if (strcmp (slp->item[j], s) == 0)
+      {
+        const char *found = slp->item[j];
+        slp->nitems--;
+        if (slp->nitems > j)
+          memmove (&slp->item[j + 1], &slp->item[j],
+                   (slp->nitems - j) * sizeof (const char *));
+        return found;
+      }
+  return NULL;
+}
