@@ -1,6 +1,5 @@
 /* xgettext C# backend.
-   Copyright (C) 2003-2004, 2006, 2014, 2018 Free Software Foundation, Inc.
-   Written by Bruno Haible <bruno@clisp.org>, 2003.
+   Copyright (C) 2003-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,6 +14,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
+/* Written by Bruno Haible.  */
+
 
 #include <stdio.h>
 
@@ -31,7 +32,7 @@ extern "C" {
   { "cs",      "C#"  },                                                 \
 
 #define SCANNERS_CSHARP \
-  { "C#",               extract_csharp,                                   \
+  { "C#",               extract_csharp, NULL,                             \
                         &flag_table_csharp, &formatstring_csharp, NULL }, \
 
 extern void extract_csharp (FILE *fp, const char *real_filename,
