@@ -1,6 +1,5 @@
 /* xgettext Emacs Lisp backend.
-   Copyright (C) 2002-2003, 2006, 2014, 2018 Free Software Foundation, Inc.
-   Written by Bruno Haible <haible@clisp.cons.org>, 2002.
+   Copyright (C) 2002-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,6 +14,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
+/* Written by Bruno Haible.  */
+
 
 #include <stdio.h>
 
@@ -31,7 +32,7 @@ extern "C" {
   { "el",        "EmacsLisp"     },                                     \
 
 #define SCANNERS_ELISP \
-  { "EmacsLisp",        extract_elisp,                                  \
+  { "EmacsLisp",        extract_elisp, NULL,                            \
                         &flag_table_elisp, &formatstring_elisp, NULL }, \
 
 /* Scan an Emacs Lisp file and add its translatable strings to mdlp.  */
