@@ -1,5 +1,5 @@
 /* Checking of messages in POT files: so-called "syntax checks".
-   Copyright (C) 2015-2023 Free Software Foundation, Inc.
+   Copyright (C) 2015-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
-/* Written by Daiki Ueno <ueno@gnu.org>.  */
+/* Written by Daiki Ueno.  */
 
 #ifndef _XG_CHECK_H
 #define _XG_CHECK_H 1
@@ -27,9 +27,9 @@ extern "C" {
 #endif
 
 
-/* Perform all syntax checks on a message list.
+/* Perform all checks on a message list.
    Return the number of errors that were seen.  */
-extern int syntax_check_message_list (message_list_ty *mlp);
+extern int xgettext_check_message_list (message_list_ty *mlp);
 
 
 #ifdef __cplusplus
